import QtQuick 2.3
import QtGraphicalEffects 1.0

Item {
    id: button
    height: root.itemHeight * 0.8

    property alias buttonText: text.text
    signal clicked

    property alias shadow: shadow
    property color colorLight: styles.buttons.colorLight
    property color colorDark: styles.buttons.colorDark
    property alias colorText: text.color
    property alias pressed: mouseArea.pressed
    property alias boldText: text.font.bold

    FlyShadow {
        id: shadow
        target: buttonRect;
        horizontalOffset: mouseArea.pressed ? 0 : -1;
        verticalOffset: mouseArea.pressed ? 0 : 1;
        color: styles.shadows.darkColor
        radius: 2
        samples: 4
    }

    Rectangle {
        id: buttonRect
        anchors.fill: parent
        border.width: styles.buttons.borderWidth; border.color: button.colorDark; radius: styles.buttons.radius
        color: mouseArea.pressed ? button.colorDark : button.colorLight
        clip: true

        ConicalGradient {
            id: conicalGradient
            anchors.fill: parent
            angle: 45
            gradient: Gradient {
                GradientStop { position: 0; color: "transparent" }
                GradientStop { position: 0.25; color: button.colorDark }
                GradientStop { position: 0.5; color: "transparent" }
                GradientStop { position: 0.75; color: button.colorDark }
                GradientStop { position: 1.0; color: "transparent" }
            }
            visible: !lowEffects
        }

        FlyLabel {
            id: text
            anchors.fill: parent
            text: "Button"
            pressed: mouseArea.pressed
            horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter
            fontSizeMode: Text.FixedSize;
            elide: Text.ElideMiddle
        }
    }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        onClicked: button.clicked()
    }
}
