import QtQuick 2.3
import "calendar.js" as CalendarUtils

Rectangle {
    id: flyCalendar

    property bool currentDayChange: false
    property alias startDate: pathView.startDate
    property alias selectedDate: selectDayHandleRect.selectedDate

    property int minimumYear: 1900 // TODO
    property int maximumYear: 2100 // TODO

    signal dateClicked(date d)

    width: parent.width / 2; height: width
    anchors.centerIn: parent
    border { width: 1; color: styles.separator.colorDark }
    color: styles.calendar.color

    Rectangle {
        id: calendarHeader
        width: parent.width - 2; height: parent.height / 8 * 2 - 1
        x: 1; y: 1
        color: styles.calendar.headerColor

        Item {
            id: monthAndYearLabel
            width: parent.width; height: parent.height * 0.5

            FlyMenuButton {
                id: prevMonthButton
                height: parent.height - 1
                image.imagePath: "qrc:/images/leftarrow.png"
                image.imageColorEnabled: true
                image.imageColor: styles.highlightColor2
                image.imageShadow: true
                onClicked: pathView.decrementCurrentIndex();
            }

            Row {
                id: monthAndYearRow
                anchors.centerIn: parent
                width: childrenRect.width; height: parent.height

                FlyLabel {
                    height: parent.height - 1
                    text: root.locale.standaloneMonthName(pathView.currentDate.getMonth(), Locale.LongFormat)
                    font.pixelSize: height * 0.5
                    color: styles.calendar.weekDayColor
                    verticalAlignment: Text.AlignVCenter
                }

                FlyMenuButton {
                    id: yearButton
                    height: parent.height - 1
                    text: Qt.formatDate(pathView.currentDate, "yyyy")
                    textColor: yearListLoader.visible ? "transparent" : styles.calendar.weekDayColor
                    checked: yearListLoader.visible
                    onClicked: {
                        if(!yearListLoader.active)
                            yearListLoader.active = true

                        yearListLoader.visible = !yearListLoader.visible
                        if(yearListLoader.visible)
                            yearListLoader.item.currentIndex = pathView.currentDate.getFullYear() - flyCalendar.minimumYear
                    }
                }
            }

            FlyMenuButton {
                id: nextMonthButton
                anchors.right: parent.right
                height: parent.height - 1
                image.imagePath: "qrc:/images/rightarrow.png"
                image.imageColorEnabled: true
                image.imageColor: styles.highlightColor2
                image.imageShadow: true
                onClicked: pathView.incrementCurrentIndex();
            }

            FlySeparator { anchors.bottom: parent.bottom }
        }

        Row {
            id: weekHeader
            anchors.top: monthAndYearLabel.bottom
            width: parent.width; height: parent.height * 0.5

            Repeater {
                id: weekRepeater
                width: parent.width; height: parent.height
                property var daysOfWeek: [Locale.Monday, Locale.Tuesday, Locale.Wednesday, Locale.Thursday,
                    Locale.Friday, Locale.Saturday, Locale.Sunday]
                model: daysOfWeek

                Text {
                    width: weekHeader.width / weekRepeater.count; height: weekRepeater.height
                    text: root.locale.dayName(modelData, Locale.ShortFormat)
                    font.pixelSize: height * 0.4
                    horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter
                    color: (index > 4) ? styles.calendar.weekendDayColor : styles.calendar.weekDayColor
                }
            }
        }
    }

    Rectangle {
        id: pathViewRect
        anchors { left: parent.left; top: calendarHeader.bottom; right: parent.right; bottom: parent.bottom }
        color: styles.calendar.color
    }
    FlyInnerShadow { target: pathViewRect }

    Repeater {
        id: separatorRepeaterVertical
        anchors.bottom: parent.bottom
        width: pathView.width; height: parent.height - calendarHeader.height / 2
        model: 6

        FlySeparator {
            horizontal: false
            height: separatorRepeaterVertical.height
            x: separatorRepeaterVertical.width / 7 * index + separatorRepeaterVertical.width / 7
            y: separatorRepeaterVertical.y
        }
    }

    Repeater {
        id: separatorRepeaterHorizontal
        width: pathView.width; height: pathView.height
        x: pathView.x; y: pathView.y
        model: 5

        FlySeparator {
            width: separatorRepeaterHorizontal.width
            y: separatorRepeaterHorizontal.y + separatorRepeaterHorizontal.height / 6 * index + separatorRepeaterHorizontal.height / 6
            x: separatorRepeaterHorizontal.x
        }
    }

    Rectangle {
        id: selectDayHandleRect

        property int selectedIndex: 0
        property date selectedDate

        x: pathView.x + selectedIndex % 7 * pathView.cellWidth
        y: pathView.y + Math.floor(selectedIndex / 7) * pathView.cellHeight + 1
        width: pathView.cellWidth - 1; height: pathView.cellHeight - 1
        color: styles.highlightColor2
        opacity: flyCalendar.currentDayChange
                 ? 0
                 : (CalendarUtils.doesYearAndMonthTheSame(pathView.currentDate, selectedDate)
                    ? 1
                    : 0)

        Behavior on opacity { NumberAnimation {} }
    }

    PathView {
        id: pathView

        property date startDate: new Date()
        property date currentDate

        property real cellWidth: width / 7
        property real cellHeight: height / 6
        property int pathHalfWidth: (model - 1) / 2 * width
        property int previousIndex: 0
        property int nextPreviousIndex: 0
        property bool firstMove: true
        property bool isFromLeftToRight: true
        property bool updateSelectedIndex: false

        signal updateMonthsShowDate(date newDate)

        anchors { left: parent.left; top: calendarHeader.bottom; right: parent.right; bottom: parent.bottom;
            leftMargin: 1; rightMargin: 1; bottomMargin: 1 }
        clip: true
        model: 5 // only negative >= 3
        preferredHighlightBegin: 0.5
        preferredHighlightEnd: 0.5
        path: Path {
            startX: -pathView.pathHalfWidth; startY: pathView.height / 2
            PathLine { x: pathView.pathHalfWidth + pathView.width; relativeY: 0 }
        }
        delegate: Component {
            FlyMonthGrid {
                width: PathView.view.width; height: PathView.view.height
            }
        }

        onCurrentIndexChanged: {
            if(firstMove) {
                firstMove = false
            } else {
                previousIndex = nextPreviousIndex
            }

            nextPreviousIndex = currentIndex

            if(currentIndex > previousIndex) {
                isFromLeftToRight = (((currentIndex === (model - 1)) && (previousIndex === 0)) ? false : true)
            } else {
                isFromLeftToRight = (((currentIndex === 0) && (previousIndex === (model - 1))) ? true : false)
            }

            //console.log(previousIndex, currentIndex, isFromLeftToRight)

            var offset = (model - 1) / 2 * (isFromLeftToRight ? 1 : -1)
            var boundaryItem = isFromLeftToRight ? itemAt(pathHalfWidth + width - 1, height / 2) : itemAt(-pathHalfWidth + 1, height / 2)
            if(boundaryItem) {
                boundaryItem.showDate = new Date(currentItem.showDate.getFullYear(), currentItem.showDate.getMonth() + offset, 15)
            } else {
                console.log("Can't find boundary item based on x,y! Trying to send signal instead.")
                // TODO get boundary index and send generateNewData(index, offset)
            }

            if(updateSelectedIndex) {
                updateSelectedIndex = false

                selectDayHandleRect.selectedIndex = CalendarUtils.weekDayIdxByLocalizedDayName(selectDayHandleRect.selectedDate)
                        + selectDayHandleRect.selectedDate.getDate()
            }
        }

        Component.onCompleted: currentDate = Qt.binding(function() { return (currentItem ? currentItem.showDate : startDate) } )

        MouseArea {
            anchors.fill: parent
            onClicked: {
                if(pathView.moving)
                    return

                var curItem = pathView.currentItem
                if(!curItem)
                    return

                var index = Math.floor(mouse.y / pathView.cellHeight) * 7 + Math.floor(mouse.x / pathView.cellWidth)
                var date = new Date(curItem.showDate)
                date.setDate(index - curItem.weekDayOfFirst)

                //console.log(mouse.x, mouse.y, index, date)

                if(flyCalendar.currentDayChange) {
                    pathView.startDate = date
                } else {
                    selectDayHandleRect.selectedIndex = index
                    selectDayHandleRect.selectedDate = date

                    root.startDetachedProcess("fly-notes", ["--create", Qt.formatDate(date, "dd.MM.yyyy")])
                }

                if(!CalendarUtils.doesYearAndMonthTheSame(date, pathView.currentDate)) {
                    if(!flyCalendar.currentDayChange)
                        pathView.updateSelectedIndex = true

                    if(index < 20) {
                        pathView.decrementCurrentIndex()
                    } else {
                        pathView.incrementCurrentIndex()
                    }
                }

                flyCalendar.dateClicked(date)
            }
        }
    }

    Rectangle {
        id: currendDayHandleRect

        property int offsize: Math.min(pathView.cellWidth, pathView.cellHeight) * 0.1
        property real newX: ((width - offsize * 2) / 7) * (weekRepeater.daysOfWeek.indexOf(pathView.startDate.getDay()))
        property real newY: -offsize + pathView.y + pathView.cellHeight *
                            Math.floor((CalendarUtils.weekDayIdxByLocalizedDayName(pathView.startDate) + pathView.startDate.getDate()) / 7)

        width: parent.width + offsize * 2; height: pathView.cellHeight + offsize * 2
        x: -offsize
        y: -offsize + pathView.y
        gradient: styles.calendar.lineHandleGradient
        border { width: 1; color: styles.calendar.lineHandleBorderColor }
        radius: 3

        Timer {
            id: animationStartTimer
            interval: 50
            repeat: false
            running: false
            onTriggered: {
                if(!handleAnimation.running)
                    handleAnimation.start()
            }
        }

        Rectangle {
            id: currendDayHandleRect_
            width: parent.width / 7 + currendDayHandleRect.offsize * 2; height: parent.height
            border { width: currendDayHandleRect.offsize * 1.5; color: styles.calendar.handleColor }
            radius: 2
            color: "transparent"
        }

        Behavior on opacity { NumberAnimation {} }

        SequentialAnimation {
            id: handleAnimation

            NumberAnimation { target: currendDayHandleRect; property: "y"; to: currendDayHandleRect.newY; duration: (currendDayHandleRect.y === currendDayHandleRect.newY) ? 0 : 250 }
            NumberAnimation { target: currendDayHandleRect_; property: "x"; to: currendDayHandleRect.newX; duration: 250 }
        }

        Component.onCompleted: {
            opacity = Qt.binding(function() { return pathView.currentItem
                                              ? (CalendarUtils.doesYearAndMonthTheSame(pathView.currentItem.showDate, pathView.startDate)
                                                 ? 1
                                                 : 0)
                                              : 1 } )
            currendDayHandleRect_.x = ((width - offsize * 2) / 7) * (weekRepeater.daysOfWeek.indexOf(pathView.startDate.getDay()))
            y = -offsize + pathView.y + pathView.cellHeight *
                                Math.floor((CalendarUtils.weekDayIdxByLocalizedDayName(pathView.startDate) + pathView.startDate.getDate()) / 7)
            onNewXChanged.connect(function() { animationStartTimer.restart() });
            onNewYChanged.connect(function() { animationStartTimer.restart() });
            handleAnimation.stopped.connect(function() { if((currendDayHandleRect_.x !== newX) || (y !== newY)) animationStartTimer.restart() })
        }
    }

    MouseArea {
        anchors.fill: parent
        enabled: yearListLoader.enabled
        onClicked: yearListLoader.switchVisibleAndSetCalendarYead()
    }

    Loader {
        id: yearListLoader

        property int selectedYear: 0

        function switchVisibleAndSetCalendarYead() {
            visible = !visible

            var newStartDate = new Date(pathView.currentDate.getFullYear(), pathView.currentDate.getMonth(), 15)
            newStartDate.setFullYear(selectedYear)

            pathView.updateMonthsShowDate(newStartDate)
        }

        x: monthAndYearRow.x + yearButton.x
        y: calendarHeader.height * 0.5
        width: yearButton.width; height: parent.height * 0.6
        active: false
        enabled: visible
        visible: false
        sourceComponent: Component { Item {
                property alias currentIndex: yearListView.currentIndex

                FlyShadow { target: yearListRect; horizontalOffset: 0; verticalOffset: 0 }
                Rectangle {
                    id: yearListRect
                    anchors.fill: parent
                    gradient: styles.calendar.yearListGradient
                }

                ListView {
                    id: yearListView
                    anchors.fill: parent; anchors.topMargin: -yearButton.height
                    model: flyCalendar.maximumYear - flyCalendar.minimumYear + 1
                    clip: true
                    highlightRangeMode: ListView.StrictlyEnforceRange
                    preferredHighlightBegin: 0
                    preferredHighlightEnd: 0.1
                    delegate: Text {
                        width: yearListView.width; height: calendarHeader.height * 0.5
                        text: index + flyCalendar.minimumYear
                        font.pixelSize: yearButton.fontSize
                        color: ListView.isCurrentItem ? styles.calendar.weekDayColor : styles.calendar.greyDayColor
                        horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter
                        elide: Text.ElideRight

                        MouseArea {
                            anchors.fill: parent
                            onClicked: { yearListView.currentIndex = index; yearListLoader.switchVisibleAndSetCalendarYead() }
                        }
                    }

                    onCurrentIndexChanged: yearListLoader.selectedYear = currentIndex + flyCalendar.minimumYear
                }
            }
        }
    }
}
