import QtQuick 2.3
import QtQuick.Layouts 1.2

Item {
    id: colorPicker

    property var colorArray: [0.3, 0.4, 0.5, 0.6] // rgba

    // TODO: need to save handleWhite.x and selectRect.rotation in settings and restore it on showing color picker

    Item {
        anchors.centerIn: parent
        width: Math.min(parent.width, parent.height); height: width

        MouseArea {
            function setColor() {
                var c = root.getColorByCoords(colorRect.image, mouseX, mouseY)

                if(c === "" || c === "0:0:0:0")
                    return

                colorArray = c.split(':')
                colorRect.baseColorArray = [colorArray[0], colorArray[1], colorArray[2]]

                var katet1 = mouseX - width / 2
                var katet2 = height / 2 - mouseY
                var angle = 90 - (Math.atan(katet1 / katet2)) * (180 / Math.PI)
                if(katet2 < 0) angle += 180
                angle *= -1

                selectRect.rotation = angle
                selectRect.x = width / 2 + (canvas.radius - width / 8) * Math.cos(angle * (Math.PI / 180))
                selectRect.y = height / 2 + (canvas.radius - width / 8) * Math.sin(angle * (Math.PI / 180))
                selectRect.visible = true

                handleWhite.x = handleWhite.parent.width / 2 - handleWhite.width / 2
            }

            anchors.fill: parent
            onClicked: {
                if(!colorRect.image) {
                    canvas.grabToImage(function(result) { colorRect.image = result.image; setColor() });
                } else {
                    setColor()
                }
            }
        }

        Image {
            anchors.fill: colorRect
            source: "qrc:/images/transparentTile.png"
            fillMode: Image.Tile
            horizontalAlignment: Image.AlignLeft; verticalAlignment: Image.AlignTop
        }

        Rectangle {
            id: colorRect

            property var image: null
            property var baseColorArray // [] rgb

            anchors { fill: parent; margins: parent.width / 4 }
            color: Qt.rgba(colorArray[0], colorArray[1], colorArray[2], colorArray[3])
            radius: width / 2

            ColumnLayout{
                anchors.fill: parent
                spacing: 4 * dpiRatio

                Item {
                    Layout.fillWidth: true; Layout.fillHeight: true

                    FlyLabel {
                        anchors.fill: parent
                        text: root.getColorString(Qt.rgba(colorArray[0], colorArray[1], colorArray[2], colorArray[3]))
                        font.pixelSize: height * 0.25;
                        color: styles.colors.whiteD
                        horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter
                        style: Text.Raised; styleColor: styles.colors.black
                    }
                }

                Item {
                    Layout.fillWidth: true; Layout.fillHeight: true

                    Image {
                        id: topRightItem
                        anchors.fill: backAlpha; anchors.margins: 1
                        source: "qrc:/images/transparentTile.png"
                        rotation: 90
                        fillMode: Image.Tile
                        horizontalAlignment: Image.AlignLeft; verticalAlignment: Image.AlignTop
                        sourceSize: Qt.size(backAlpha.width, backAlpha.width)
                    }

                    Rectangle {
                        id: backAlpha
                        anchors.centerIn: parent; anchors.verticalCenterOffset: -parent.height * 0.3
                        height: parent.width * 0.8; width: parent.height * 0.4
                        rotation: 90
                        border { width: 1; color: styles.colors.black4 }
                        gradient: Gradient {
                            GradientStop { position: 0.0; color: "transparent" }
                            GradientStop { position: 1.0; color: Qt.rgba(colorArray[0], colorArray[1], colorArray[2], 1) }
                        }
                    }
                    MouseArea {
                        anchors.horizontalCenter: parent.horizontalCenter
                        width: parent.width * 0.8; height: parent.height * 0.4
                        onClicked: { handleAlpha.x = mouse.x + x - handleAlpha.width / 2; handleAlpha.updateAlpha() }
                    }

                    Rectangle {
                        id: handleAlpha
                        property real origX: parent.width / 2 - backAlpha.height / 2 + 1
                        x: origX
                        width: 4; height: backAlpha.width
                        color: "transparent"
                        border { width: 1; color: styles.colors.black4 }

                        function updateAlpha() {
                            var width = mouseAreaAlpha.drag.maximumX - mouseAreaAlpha.drag.minimumX
                            var x = handleAlpha.x - origX
                            colorPicker.colorArray[3] = 1.0 - x / width
                            colorPicker.colorArrayChanged()
                        }

                        MouseArea {
                            id: mouseAreaAlpha
                            anchors { fill: parent; leftMargin: -15 * dpiRatio; rightMargin: -15 * dpiRatio }
                            drag {
                                target: parent
                                axis: Drag.XAxis
                                minimumX: handleAlpha.parent.width / 2 - backAlpha.height / 2 + 1
                                maximumX: handleAlpha.parent.width / 2 + backAlpha.height / 2 - handleAlpha.width
                            }
                        }
                        onXChanged: if(mouseAreaAlpha.pressed) updateAlpha()

                        Component.onCompleted: x = origX + (-backAlpha.height * (colorArray[3] - 1.0))
                    }

                    Rectangle {
                        id: backWhite
                        anchors.centerIn: parent; anchors.verticalCenterOffset: parent.height * 0.3
                        height: parent.width * 0.8; width: parent.height * 0.4
                        rotation: 90
                        border { width: 1; color: styles.colors.black4 }
                        gradient: Gradient {
                            GradientStop { position: 0.0; color: "white" }
                            GradientStop { position: 0.5; color: Qt.rgba(colorRect.baseColorArray[0], colorRect.baseColorArray[1],
                                                                         colorRect.baseColorArray[2], 1) }
                            GradientStop { position: 1.0; color: "black" }
                        }
                    }
                    MouseArea {
                        anchors { horizontalCenter: parent.horizontalCenter; bottom: parent.bottom }
                        width: parent.width * 0.8; height: parent.height * 0.4
                        onClicked: { handleWhite.x = mouse.x + x - handleWhite.width / 2; handleWhite.updateWhite() }
                    }

                    Rectangle {
                        id: handleWhite
                        anchors.bottom: parent.bottom
                        x: parent.width / 2 - width / 2
                        width: 4; height: backWhite.width
                        color: "transparent"
                        border { width: 1; color: styles.colors.black4 }

                        function updateWhite() {
                            var width = mouseAreaWhite.drag.maximumX - mouseAreaWhite.drag.minimumX
                            var x = handleWhite.x - (parent.width / 2 - backWhite.height / 2 + 1)
                            var step = x / width / 0.5
                            if(step < 1.0) {
                                colorPicker.colorArray[0] = step * colorRect.baseColorArray[0]
                                colorPicker.colorArray[1] = step * colorRect.baseColorArray[1]
                                colorPicker.colorArray[2] = step * colorRect.baseColorArray[2]
                                colorPicker.colorArrayChanged()
                            } else {
                                step -= 1.0
                                // he thinks that colorRect.baseColorArray[n] is a string -_-
                                colorPicker.colorArray[0] = 1 * colorRect.baseColorArray[0] + step * (1 - colorRect.baseColorArray[0])
                                colorPicker.colorArray[1] = 1 * colorRect.baseColorArray[1] + step * (1 - colorRect.baseColorArray[1])
                                colorPicker.colorArray[2] = 1 * colorRect.baseColorArray[2] + step * (1 - colorRect.baseColorArray[2])
                                colorPicker.colorArrayChanged()
                            }
                        }

                        MouseArea {
                            id: mouseAreaWhite
                            anchors { fill: parent; leftMargin: -15 * dpiRatio; rightMargin: -15 * dpiRatio }
                            drag {
                                target: parent
                                axis: Drag.XAxis
                                minimumX: handleWhite.parent.width / 2 - backWhite.height / 2 + 1
                                maximumX: handleWhite.parent.width / 2 + backWhite.height / 2 - handleWhite.width
                            }
                        }
                        onXChanged: if(mouseAreaWhite.pressed) updateWhite()
                    }
                }

                Item { Layout.fillWidth: true; Layout.fillHeight: true }
            }

            Component.onCompleted: baseColorArray = [colorArray[0], colorArray[1], colorArray[2]]
        }

        FlyShadow { target: canvas; color: styles.shadows.darkColor }
        Canvas {
            id: canvas

            property real radius: width / 2 - width / 8

            anchors.fill: parent
            onPaint: {
                var ctx = getContext("2d")

                function drawMultiRadiantCircle(xc, yc, r, radientColors) {
                    var partLength = (2 * Math.PI) / radientColors.length;
                    var start = 0;
                    var gradient = null;
                    var startColor = null,
                            endColor = null;

                    for (var i = 0; i < radientColors.length; i++) {
                        startColor = radientColors[i];
                        endColor = radientColors[(i + 1) % radientColors.length];

                        // x start / end of the next arc to draw
                        var xStart = xc + Math.cos(start) * r;
                        var xEnd = xc + Math.cos(start + partLength) * r;
                        // y start / end of the next arc to draw
                        var yStart = yc + Math.sin(start) * r;
                        var yEnd = yc + Math.sin(start + partLength) * r;

                        ctx.beginPath();

                        gradient = ctx.createLinearGradient(xStart, yStart, xEnd, yEnd);
                        gradient.addColorStop(0, startColor);
                        gradient.addColorStop(1.0, endColor);

                        ctx.strokeStyle = gradient;
                        ctx.arc(xc, yc, r, start, start + partLength + Math.PI / 360);
                        ctx.lineWidth = width / 4;
                        ctx.stroke();
                        ctx.closePath();

                        start += partLength;
                    }
                }

                var someColors = [];
                someColors.push('#F00');
                someColors.push('#FF0');
                someColors.push('#0F0');
                someColors.push('#0FF');
                someColors.push('#00F');
                someColors.push('#F0F');
                someColors.push('#FFF');

                drawMultiRadiantCircle(width / 2, height / 2, radius, someColors);
            }
        }

        Rectangle {
            id: selectRect
            width: canvas.width / 4; height: 4
            x: parent.width / 2
            color: "transparent"
            border { width: 1; color: styles.colors.black }
            transformOrigin: Item.Left
            antialiasing: true
            visible: false
        }
    }
}
