import QtQuick 2.3
import QtGraphicalEffects 1.0

Item {
    id: coloredImage

    property alias imagePath: image.source
    property color imageColor: pressed || !enabled ? styles.colors.whiteE : styles.colors.white
    property alias imageColorEnabled: imageColorOverlayLoader.active
    property alias imageShadow: imageShadowLoader.active
    property alias imageFillMode: image.fillMode
    property alias imageSourceSize: image.sourceSize
    property alias imageDesaturateEnabled: imageDesaturateLoader.active
    property alias imageAntialiasing: image.antialiasing
    property real  imageDesaturation: 1.0

    property string subImagePath: ""
    property color  subImageColor: pressed ? root.getColorString(0xAA, styles.colors.whiteC) : root.getColorString(0x55, styles.colors.white)
    property alias  subImageColorEnabled: subImageColorOverlayLoader.active
    property alias  subImageShadow: subImageShadowLoader.active

    property bool pressed: false

    width: root.itemHeight; height: width
    visible: image.source != ""

    Loader { id: imageShadowLoader; anchors.fill: image; active: false; sourceComponent: imageShadowComponent }
    Component {
        id: imageShadowComponent

        FlyShadow {
            target: image;
            anchors.fill: undefined
            horizontalOffset: -1;
            verticalOffset: 1;
            visible: coloredImage.pressed ? false : (target.visible && !lowEffects)
            radius: 2; samples: 4
        }
    }

    Image {
        id: image
        anchors.fill: parent
        //transformOrigin: Item.BottomRight
        scale: coloredImage.pressed ? !root.isDesktopMode ? 1.1 : 0.97 : 1.0
        mipmap: styles.image.mipmap
    }

    Loader { id: imageColorOverlayLoader; anchors.fill: image; active: false; sourceComponent: imageColorOverlayComponent }
    Component {
        id: imageColorOverlayComponent

        ColorOverlay {
            id: imageColorOverlay
            source: image
            scale: image.scale
            transformOrigin: image.transformOrigin
            rotation: image.rotation
            color: coloredImage.imageColor
            visible: !lowEffects
        }
    }

    Loader { id: imageDesaturateLoader; anchors.fill: image; active: false; sourceComponent: imageDesaturateComponent }
    Component {
        id: imageDesaturateComponent

        Desaturate {
            id: imageDesaturate
            source: image
            scale: image.scale
            transformOrigin: image.transformOrigin
            rotation: image.rotation
            desaturation: coloredImage.imageDesaturation
            cached: true
        }
    }

    Loader { id: subImageShadowLoader; anchors.fill: subImageLoader; active: false; sourceComponent: subImageShadowComponent }
    Component {
        id: subImageShadowComponent

        FlyShadow {
            id: subImageShadow
            target: subImageLoader.item ? subImageLoader.item : subImageLoader
            anchors.fill: undefined
            horizontalOffset: coloredImage.pressed ? 0 : -1;
            verticalOffset: coloredImage.pressed ? 0 : 1;
            radius: 2; samples: 4
            visible: !lowEffects
        }
    }

    Loader { id: subImageLoader; anchors { left: image.horizontalCenter; top: image.verticalCenter; right: image.right; bottom: image.bottom }
        active: coloredImage.subImagePath != ""; sourceComponent: subImageComponent }
    Component {
        id: subImageComponent

        Image {
            id: subImage
            scale: coloredImage.pressed ? !root.isDesktopMode ? 1.1 : 0.97 : 1.0
            source: coloredImage.subImagePath
            sourceSize: Qt.size(coloredImage.height, coloredImage.height)
            mipmap: styles.image.mipmap
        }
    }

    Loader { id: subImageColorOverlayLoader; anchors.fill: subImageLoader; active: false; sourceComponent: subImageColorOverlayComponent }
    Component {
        id: subImageColorOverlayComponent

        ColorOverlay {
            id: subImageColorOverlay
            source: subImageLoader.item ? subImageLoader.item : subImageLoader
            color: coloredImage.subImageColor
            visible: !lowEffects
        }
    }
}
