import QtQuick 2.3
import Fly.Components 1.0

Item {
    id: contactFullInfo

    property var contact
    property var confirmFunction
    property string photoPath
    property bool showNameOnPhoto: false
    signal photoChosen(string photoPath)

    property alias phonesRectShadowVisible: phonesRectShadow.visible
    readonly property alias contactPropertyList: contactPropertyList

    readonly property bool horizontal: width > height + height * 0.1
    readonly property real margin: 5 * dpiRatio

    function init(contact, contactText) {
        contactFullInfo.contact = contact
        photoButton.source = "image://ContactImageProvider/" + (contact ? contact.id + '@' + contact.photoVersion : "unknown")
        photoPath = ""

        contactPropertyList.init(contact, contactText)
    }

    function saveContact() {
        contactPropertyList.saveYourself()

        var contact = contactFullInfo.contact
        if(!contact)
            contact = db.getContact()

        contact.name = contactPropertyList.nameText
        contact.surname = contactPropertyList.surnameText
        contact.middlename = contactPropertyList.middlenameText
        contact.organization = contactPropertyList.organizationText
        contact.dateOfBirth = contactPropertyList.dateOfBirthText
        contact.comment = contactPropertyList.comments
        contact.phoneList = contactPropertyList.phoneList
        contact.phoneGroupList = contactPropertyList.phoneNameList
        contact.emailList = contactPropertyList.emails
        contact.emailGroupList = contactPropertyList.emailNames
        contact.addressList = contactPropertyList.addresses
        contact.addressGroupList = contactPropertyList.addressNames
        contact.linkList = contactPropertyList.links

        if(contactFullInfo.contact) {
            db.updateContact(contact)
        } else {
            db.addContact(contact)
        }

        if(photoPath !== "") {
            root.setContactPhoto(contact.id, photoPath)
            contact.photoVersion++;
        }

        if(confirmFunction) confirmFunction(contact.id)

        clearMemory()
    }

    function clearMemory() {
        if(contact)
            contact.deleteLater()
    }

    FlyPhotoButton {
        id: photoButton
        width: contactFullInfo.horizontal ? parent.width * 0.5 - contactFullInfo.margin : parent.width
        height: contactFullInfo.horizontal ? parent.height : parent.height * 0.5 - (phonesRectShadow.visible ? contactFullInfo.margin : 0)
        margins: 0
        z: 1

        function selectMenu(index) {
            if(index === 0) {
                if(debug) console.log("Launching camera app...");
               root.startDetachedProcess("fly-camera", []);
            } else if (index === 1) {
                if(debug) console.log("Choosing existing file...");
                var fileName = FlyFileDialog.chooseImageFile();
                if(fileName) {
                    if(debug) console.log("File chosen: ", fileName);

                    photoButton.source = "file://" + fileName;
                    contactFullInfo.photoPath = fileName;
                    contactFullInfo.photoChosen(fileName);
                }
            }
        }

        Rectangle {
            id: nameRect
            visible: showNameOnPhoto && !!contact && contact.fullname
            anchors.bottom: parent.bottom
            width: parent.width
            height: fullnameLabel.height + fullnameLabel.anchors.margins * 2
            color: root.getColorString(0x88, styles.colors.black)

            FlyLabel {
                id: fullnameLabel
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 10 * dpiRatio }
                font.pixelSize: styles.label.big; fontSizeMode: Text.FixedSize;
                elide: Text.ElideRight
                wrapMode: Text.Wrap
                color: styles.colors.white
                text: contact ? contact.fullname : ""
                style: Text.Raised; styleColor: nameRect.color
            }
        }

        onClicked: {
            root.showListViewPage({
                                      "title": qsTr("Photo menu:"),
                                      "model": [qsTr("Take a picture"), qsTr("Choose existing picture...")],
                                      "okFunction": selectMenu
                                  })
        }
    }

    Rectangle {
        id: phonesRect
        x: contactFullInfo.horizontal ? parent.width * 0.5 + contactFullInfo.margin : 0
        y: contactFullInfo.horizontal ? 0 : parent.height * 0.5 + (phonesRectShadow.visible ? contactFullInfo.margin : 0)
        width: contactFullInfo.horizontal ? parent.width * 0.5 - contactFullInfo.margin : parent.width
        height: contactFullInfo.horizontal ? parent.height : parent.height * 0.5 - (phonesRectShadow.visible ? contactFullInfo.margin : 0)
        color: styles.colors.white
    }

    FlyInnerShadow { id: phonesRectShadow; target: phonesRect }

    FlyContactPropertyList {
        id: contactPropertyList
        anchors { fill: phonesRect; leftMargin: 10 * dpiRatio; rightMargin: anchors.leftMargin }
    }
}
