import QtQuick 2.3

FocusScope {
    id: contactList

    property bool emptyList: (contactsListView.list.count === 0)
    property var  model: []  // should provide idString, name, phones, emails, customData
    property bool showPhones: true // if not then emails
    property bool showFullname: false
    property bool popupMode: true
    property bool chooseOnePhone: false
    property bool showContactInfoViaPhotoClick: false
    property alias list: contactsListView.list
    signal clicked(string i, string n, string p, string c, int index)

    readonly property bool checkMode: state === "checkMode"
    property var checkedContacts: []

    width: parent.width; height: centerRect.height * 0.5
    visible: !popupMode
    enabled: visible
    z: 1000

    states: State {
        name: "checkMode"
        StateChangeScript { script: checkedContacts = []; }
    }

    function show(buttonClicked) {
        contactsListView.list.contentY = 0
        visible = buttonClicked ? true : Qt.binding(function() { return !contactList.emptyList } )
    }

    function hide() {
        visible = false
    }

    function toggleContactToRemove(id) {
        var index = checkedContacts.indexOf(id);
        if(index >= 0) { checkedContacts.splice(index, 1); } else { checkedContacts.push(id); }
        checkedContactsChanged();
    }

    function toggleCheckMode() {
        state = checkMode ? "" : "checkMode";
    }

    MouseArea {
        x: root.currentPage ? -mapToItem(root.currentPage, 0, 0).x : 0
        y: root.currentPage ? -mapToItem(root.currentPage, 0, 0).y : 0
        width: root.width; height: root.height
        enabled: contactList.popupMode
        onClicked: contactList.hide()
    }

    FlyShadow { target: listRect; horizontalOffset: -2; verticalOffset: 2; visible: contactList.popupMode }
    Rectangle {
        id: listRect
        anchors.fill: parent
        color: contactList.popupMode ? styles.colors.white : "transparent"
        border { color: Qt.darker(color); width: contactList.popupMode ? 1 : 0 }
        radius: contactList.popupMode ? 2 * dpiRatio : 0

        FlyListView {
            id: contactsListView
            anchors.fill: parent; anchors.margins: contactList.popupMode ? 1 : 0
            list.model: contactList.model
            list.currentIndex: -1
            list.delegate: Item {
                id: listItem

                readonly property bool isChecked: checkedContacts.indexOf(idString) >= 0
                property var dataList: contactList.showPhones ? contact.phoneList : contact.emailList
                property var dataName: contactList.showFullname ? fullname : name

                width: contactsListView.width; height: root.itemHeight * 1.5
                clip: true

                FlyTouchAnimation { mouseArea: listItemMouseArea }

                FlyColoredImage {
                    id: attachImage
                    x: parent.height * 0.1; width: height; height: parent.height * 0.8
                    anchors.verticalCenter: parent.verticalCenter
                    pressed: photoMouseArea.enabled ? photoMouseArea.pressed : listItemMouseArea.pressed
                    imagePath: "image://ContactImageProvider/" + contact.id  + '@' + contact.photoVersion
                }

                FlyLabel {
                    id: listItemWhoLabel
                    anchors { left: attachImage.right; top: parent.top; bottom: parent.verticalCenter
                        leftMargin: parent.height * 0.1 }
                    width: Math.min(implicitWidth, parent.width - x)
                    text: dataName
                    font.pixelSize: styles.label.big; fontSizeMode: Text.FixedSize;
                    verticalAlignment: Text.AlignVCenter
                    elide: Text.ElideRight
                }

                FlyLabel {
                    anchors { left: attachImage.right; top: parent.verticalCenter; bottom: parent.bottom
                        leftMargin: parent.height * 0.1 }
                    width: Math.min(implicitWidth, parent.width - x)
                    text: dataList.join(", ")
                    font.pixelSize: styles.label.smallest; fontSizeMode: Text.FixedSize;
                    verticalAlignment: Text.AlignVCenter
                    elide: Text.ElideRight
                }

                Rectangle {
                    id: checkRect
                    anchors.fill: parent
                    color: root.getColorString(0xAA, styles.colors.black)
                    visible: checkMode

                    Rectangle {
                        anchors.centerIn: parent
                        width: parent.height * 0.5; height: width
                        color: root.getColorString(0x88, styles.colors.white)
                        border { width: 1; color: styles.colors.whiteC }
                        radius: 3

                        Text {
                            anchors.fill: parent
                            text: "\u2713"
                            color: styles.colors.black
                            font.pixelSize: height
                            horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter
                            visible: listItem.isChecked ? 1.0 : 0.0
                        }
                    }
                }

                FlySeparator { anchors.bottom: parent.bottom }

                MouseArea {
                    id: listItemMouseArea
                    anchors.fill: parent
                    onClicked: {
                        if(checkMode) { toggleContactToRemove(idString); return; }

                        if(dataList.length > 1) {
                            if(contactList.popupMode || contactList.chooseOnePhone) {
                                root.showListViewPage({ "title": qsTr("Choose item:"),
                                                          "model": dataList,
                                                          "okFunction": function(i) { contactList.clicked(idString, dataName, dataList[i], customData, index);
                                                              if(contactList.popupMode) contactList.hide() } })
                            } else {
                                contactList.clicked(idString, dataName, "", "", index)
                            }
                        } else {
                            contactList.clicked(idString, dataName, dataList, customData, index)

                            if(contactList.popupMode)
                                contactList.hide()
                        }
                    }
                    onPressAndHold: { toggleCheckMode(); if(checkMode) toggleContactToRemove(idString); }
                }

                MouseArea {
                    id: photoMouseArea
                    anchors { left: parent.left; top: parent.top; right: listItemWhoLabel.left; bottom: parent.bottom }
                    enabled: contactList.showContactInfoViaPhotoClick && !checkMode
                    onClicked: root.showContactShortInfoPage(idString)
                }
            }
        }
    }

    FlyLabel {
        anchors.fill: parent
        text: qsTr("Contacts not found.")
        font.pixelSize: styles.label.big
        visible: contactList.emptyList
        wrapMode: Text.WordWrap
        horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter
    }
}
