import QtQuick 2.3

Column {
    id: detailsGroupEdit

    property var detailsData: []
    property var detailsName: []
    property alias groupText: groupLabel.text
    property int inputMethodHints: 0
    property string placeHolderText: "+7 012 345-67-89"
    property bool withGroup: true
    signal saveYourself

    FlyGroupLabel {
        id: groupLabel
        width: parent.width; height: root.itemHeight;
        onClicked: {
            detailsGroupEdit.saveYourself()
            detailsGroupEdit.detailsName.push((text === qsTr("PHONE")) ? qsTr("Mobile") : qsTr("Home"))
            detailsGroupEdit.detailsData.push("")
            detailsGroupEdit.detailsData = detailsGroupEdit.detailsData // for Repeater model update
        }
    }

    Binding {
        target: groupLabel
        property: "withButton"
        value: true
        when: phones.model === 0
    }

    Repeater {
        id: phones
        width: parent.width; model: detailsGroupEdit.detailsData.length

        Item {
            width: detailsGroupEdit.width; height: root.itemHeight

            FlyLineEdit {
                id: phone
                width: detailsGroupEdit.withGroup ? parent.width * 0.6 : parent.width - removeLineButton.width
                placeholderText: detailsGroupEdit.placeHolderText
                text: detailsGroupEdit.detailsData[index] ? detailsGroupEdit.detailsData[index] : ""
                showClearButton: false
                inputMethodHints: detailsGroupEdit.inputMethodHints
                regExp: (groupLabel.text === qsTr("PHONE"))
                        ? /^((8|\+7)[\- ]?)?(\(?\d{3}\)?[\- ]?)?[\d\- ]{7,10}$/
                        : /.*/

                Binding {
                    target: groupLabel
                    property: "withButton"
                    value: phone.text !== ""
                    when: index === detailsGroupEdit.detailsData.length - 1
                }
            }

            FlyButton {
                id: groupButton

                property var extGroups: []

                function getGroups() {
                    var groups = db.getGroups()
                    extGroups = groups.map(function(item, index) { return [item, (index > 2 ? true : false)] } )
                    extGroups.push([qsTr("Create new group..."), false])

                    return extGroups
                }

                function setGroup(index, itemString) {
                    if(itemString === "") return

                    if(index === extGroups.length - 1) {
                        root.showLineEditPage({
                                                  "title": qsTr("New group:"),
                                                  "line.text": "",
                                                  "selectFunction": newGroup,
                                                  "line.placeholderText": qsTr("Group name") })
                    } else {
                        buttonText = itemString
                    }
                }

                function newGroup(group) {
                    if(group === "") return

                    if(!extGroups.some(function(item) { return item[0] === group }))
                        db.addGroup(group)

                    setGroup(extGroups.length, group)
                }

                anchors { left: phone.right; top: parent.top; right: removeLineButton.left; bottom: parent.bottom;
                    leftMargin: 10; topMargin: 10 }
                buttonText: detailsGroupEdit.detailsName[index]
                            ? detailsGroupEdit.detailsName[index]
                            : ((groupLabel.text === qsTr("PHONE")) ? qsTr("Mobile") : qsTr("Home"))
                visible: detailsGroupEdit.withGroup
                onClicked: {
                    root.showListViewPage({
                                              "title": qsTr("Choose group:"),
                                              "model": getGroups(),
                                              "okFunction": setGroup,
                                              "deleteMode": true
                                          })
                    root.currentPage.onItemDeleted.connect(function(index, group) { extGroups.splice(index, 1); db.removeGroup(group) })
                }
            }

            Text {
                id: removeLineButton
                anchors { top: parent.top; right: parent.right; bottom: parent.bottom; topMargin: 10 }
                width: 30 * dpiRatio
                color: styles.label.color
                text: "\u2717"
                font.pixelSize: styles.label.big; fontSizeMode: Text.HorizontalFit
                horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        detailsGroupEdit.saveYourself()
                        if(detailsGroupEdit.withGroup)
                            detailsGroupEdit.detailsName.splice(index, 1)
                        detailsGroupEdit.detailsData.splice(index, 1)
                        detailsGroupEdit.detailsData = detailsGroupEdit.detailsData // for Repeater model update
                    }
                }
            }

            Connections {
                target: detailsGroupEdit
                onSaveYourself: {
                    if(index >= detailsGroupEdit.detailsData.length) return

                    detailsGroupEdit.detailsData[index] = phone.text
                    if(detailsGroupEdit.withGroup) {
                        detailsGroupEdit.detailsName[index] = groupButton.buttonText
                    }
                }
            }
        }
    }
}
