import QtQuick 2.3

Item {
    id: contactDetailsEdit

    property alias nameText: name.text
    property alias surnameText: surname.text
    property alias middlenameText: middlename.text
    property alias organizationText: organization.text
    property alias dateOfBirthText: dateOfBirth.text
    property alias phoneList: phoneDetailsGroup.detailsData
    property alias phoneNameList: phoneDetailsGroup.detailsName
    property alias emails: emailDetailsGroup.detailsData
    property alias emailNames: emailDetailsGroup.detailsName
    property alias addresses: addressDetailsGroup.detailsData
    property alias addressNames: addressDetailsGroup.detailsName
    property alias links: linkDetailsGroup.detailsData
    property alias comments: comment.text

    function init(contact, contactText) {
        if(contact) {
            nameText = contact.name
            surnameText = contact.surname
            middlenameText = contact.middlename
            organizationText = contact.organization
            dateOfBirthText = contact.dateOfBirth
            comments = contact.comment
            phoneList = []
            phoneNameList = []
            emails = contact.emailList
            emailNames = contact.emailGroupList
            addresses = contact.addressList
            addressNames = contact.addressGroupList
            links = contact.linkList

            if(contactText) {
                phoneList = phoneList.concat(contact.phoneList, contactText)
                phoneNameList = phoneNameList.concat(contact.phoneGroupList, qsTr("Mobile"))
            } else {
                phoneList = phoneList.concat(contact.phoneList)
                phoneNameList = phoneNameList.concat(contact.phoneGroupList)
            }
        } else {
            nameText = ""
            surnameText = ""
            middlenameText = ""
            organizationText = ""
            dateOfBirthText = ""
            comments = ""
            phoneList = [contactText]
            phoneNameList = [qsTr("Mobile")]
            emails = []
            emailNames = []
            addresses = []
            addressNames = []
            links = []
        }

        detailsFlickable.contentY = 0
    }

    function saveYourself() {
        phoneDetailsGroup.saveYourself()
        emailDetailsGroup.saveYourself()
        addressDetailsGroup.saveYourself()
        linkDetailsGroup.saveYourself()
    }

    Flickable {
        id: detailsFlickable
        anchors.fill: parent
        contentWidth: contentItem.childrenRect.width; contentHeight: contentItem.childrenRect.height
        flickableDirection: Flickable.VerticalFlick
        clip: true

        Column {
            FlyGroupLabel { width: detailsFlickable.width; height: root.itemHeight; text: qsTr("NAME") }
            FlyLineEdit { id: name; width: detailsFlickable.width; placeholderText: qsTr("Name") }
            FlyLineEdit { id: surname; width: detailsFlickable.width; placeholderText: qsTr("Surname") }
            FlyLineEdit { id: middlename; width: detailsFlickable.width; placeholderText: qsTr("Middlename") }

            FlyContactPropertyGroup { id: phoneDetailsGroup; width: detailsFlickable.width; groupText: qsTr("PHONE")
                inputMethodHints: Qt.ImhDialableCharactersOnly }

            FlyGroupLabel { width: detailsFlickable.width; height: root.itemHeight; text: qsTr("PRIVATE") }
            Row {
                spacing: 10

                function onSelectDate(date) {
                    var day = (date.getDate() < 10) ? "0" + date.getDate() : date.getDate()
                    var month = ((date.getMonth() + 1) < 10) ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1)
                    dateOfBirth.text = day + "." + month + "." + date.getFullYear()
                }

                FlyLineEdit { id: dateOfBirth; width: detailsFlickable.width - calendarButton.width - parent.spacing
                    placeholderText: qsTr("Date of birth (01.12.1980)"); regExp: /^(0[1-9]|[12][0-9]|3[01])\.(0[1-9]|1[012])\.(19|20)\d\d$/ }
                FlyPhotoButton { id: calendarButton; height: root.itemHeight; width: height; source: "qrc:/images/calendar.png";
                    onClicked: root.showCalendar({ "title": qsTr("Choose your date of birth"),
                                                     "currentDayChange": true,
                                                     "startDate": (dateOfBirth.text ? dateOfBirth.text.split(".").reverse().join("-") : new Date()),
                                                     "selectFunction": parent.onSelectDate } ) }
            }
            FlyLineEdit { id: organization; width: detailsFlickable.width; placeholderText: qsTr("Organization") }

            FlyContactPropertyGroup { id: emailDetailsGroup; width: detailsFlickable.width; groupText: qsTr("EMAIL"); placeHolderText: "user@mail.com"
                inputMethodHints: Qt.ImhEmailCharactersOnly }
            FlyContactPropertyGroup { id: addressDetailsGroup; width: detailsFlickable.width; groupText: qsTr("ADDRESS"); placeHolderText: qsTr("Address") }
            FlyContactPropertyGroup { id: linkDetailsGroup; width: detailsFlickable.width; groupText: qsTr("LINKS")
                placeHolderText: "http://www.rusbitech.ru/"; withGroup: false; inputMethodHints: Qt.ImhUrlCharactersOnly }

            FlyGroupLabel { width: detailsFlickable.width; height: root.itemHeight; text: qsTr("COMMENT") }
            FlyLineEdit { id: comment; width: detailsFlickable.width; placeholderText: qsTr("Additional info") }

            Item { width: detailsFlickable.width; height: root.itemHeight }
        }
    }
}
