import QtQuick 2.3
import Fly.Components 1.0

FlyPage {
    id: contactInfoPage

    property var contact

    title: qsTr("Contact info")
    titleImage.imagePath: "qrc:/images/contact.png"
    titleImage.imageColorEnabled: false
    titleImage.imageShadow: false
    showCancelButton: false
    leftMargin: parent ? parent.width * root.pageMargin : 0
    topMargin: parent ? parent.height * root.pageMargin : 0
    okFunction: clearMemory
    cancelFunction: clearMemory

    function init(contact) {
        contactInfoPage.contact = contact
        phonesListView.list.model = []

        fullnameLabel.text = contact.fullname
        photoButton.source = "image://ContactImageProvider/" + contact.id + '@' + contact.photoVersion
        phonesListView.phoneGroups = contact.phoneGroupList
        phonesListView.list.model = contact.phoneList
    }

    function clearMemory() {
        if(contact)
            contact.deleteLater()
    }

    FlyCallDbusNotifier { id: callDbusNotifier }

    Item {
        id: backgroundItem
        anchors.fill: centerRect;

        readonly property bool horizontal: width > height + height * 0.1
        readonly property real margin: 5 * dpiRatio

        FlyPhotoButton {
            id: photoButton
            width: backgroundItem.horizontal ? parent.width * 0.5 - backgroundItem.margin : parent.width
            height: backgroundItem.horizontal ? parent.height : parent.height * 0.5 - backgroundItem.margin
            imageFillMode: Image.PreserveAspectCrop
            margins: 0

            Rectangle {
                id: nameRect
                anchors.bottom: parent.bottom
                width: parent.width
                height: fullnameLabel.height + fullnameLabel.anchors.margins * 2
                color: root.getColorString(0xAA, styles.colors.black)
            }

            FlyLabel {
                id: fullnameLabel
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 10 * dpiRatio }
                font.pixelSize: styles.label.big; fontSizeMode: Text.FixedSize;
                elide: Text.ElideRight
                wrapMode: Text.Wrap
                color: styles.colors.white
                text: ""
            }

            onClicked: root.startDetachedProcess("fly-contacts", ["--contact-id", contactInfoPage.contact.id])
        }

        Rectangle {
            id: phonesRect
            x: backgroundItem.horizontal ? parent.width * 0.5 + backgroundItem.margin : 0
            y: backgroundItem.horizontal ? 0 : parent.height * 0.5 + backgroundItem.margin
            width: backgroundItem.horizontal ? parent.width * 0.5 - backgroundItem.margin : parent.width
            height: backgroundItem.horizontal ? parent.height : parent.height * 0.5 - backgroundItem.margin
            color: styles.colors.white
        }

        FlyInnerShadow { target: phonesRect }

        FlyRadioButtonRow {
            id: radioButtonRow
            anchors { left: phonesRect.left; top: phonesRect.top; right: phonesRect.right; margins: 10 * dpiRatio }
            height: root.itemHeight
            model: [qsTr("Phone call"), qsTr("Send SMS")]
            currentIndex: 0 // TODO from settings
        }

        FlyListView {
            id: phonesListView

            property var phoneGroups: []

            anchors { left: phonesRect.left; top: radioButtonRow.bottom; right: phonesRect.right; bottom: phonesRect.bottom;
                margins: backgroundItem.margin }
            list.model: []
            list.delegate: Item {
                id: listItem
                width: ListView.view.width; height: root.itemHeight * 1.5
                clip: true

                FlyTouchAnimation { mouseArea: listItemMouseArea }

                FlyLabel {
                    id: phoneLabel
                    x: 10 * dpiRatio
                    anchors { top: parent.top; bottom: parent.verticalCenter; topMargin: 5 * dpiRatio }
                    width: Math.min(implicitWidth, parent.width)
                    pressed: listItemMouseArea.pressed
                    text: modelData
                    font.pixelSize: styles.label.big; fontSizeMode: Text.FixedSize;
                    verticalAlignment: Text.AlignVCenter
                    elide: Text.ElideRight
                }

                FlyLabel {
                    id: groupLabel
                    x: 10 * dpiRatio
                    anchors { top: parent.verticalCenter; bottom: parent.bottom; bottomMargin: 5 * dpiRatio }
                    width: Math.min(implicitWidth, parent.width)
                    pressed: listItemMouseArea.pressed
                    text: phonesListView.phoneGroups[index]
                    font.pixelSize: styles.label.big; fontSizeMode: Text.FixedSize;
                    verticalAlignment: Text.AlignVCenter
                    elide: Text.ElideRight
                    enabled: false
                }

                FlySeparator { anchors.bottom: parent.bottom }

                MouseArea {
                    id: listItemMouseArea
                    anchors.fill: parent
                    onClicked: {
                        if(radioButtonRow.currentIndex === 0) { // call
                            callDbusNotifier.emitDial(modelData)
                        } else { // sms
                            root.startDetachedProcess("fly-sms", ["--send-sms", modelData])
                        }

                        if(contactInfoPage.okFunction) contactInfoPage.okFunction(); root.removePage(contactInfoPage.pageIndex)
                    }
                }
            }
        }
    }
}
