import QtQuick 2.3
import Fly.Components 1.0

Item {
    id: contextMenu

    property alias menuRect: menuRect
    property alias model: listView.model

    property var clicked: null

    enabled: state === "visible"
    z: 1

    states: State {
        name: "visible"
        PropertyChanges { target: menuRect; width: Math.min(contextMenu.width, dummyLabel.contentWidth + listView.itemHeight);
            height: Math.min(contextMenu.height - menuRect.y, listView.count * listView.itemHeight) }
    }
    transitions: Transition {
        NumberAnimation { property: "width" }
        NumberAnimation { property: "height"; duration: 350; easing.type: Easing.InCubic }
    }

    function show(model, clicked) {
        if(model)   contextMenu.model = model;
        if(clicked) contextMenu.clicked = clicked;

        if(root.currentPage) contextMenu.parent = currentPage;

        state = "visible";
    }

    MouseArea { anchors.fill: parent; onClicked: contextMenu.state = "" }

    FlySeparator { anchors { top: menuRect.top; right: parent.right } width: menuRect.width; triple: true; z: 1; visible: contextMenu.enabled }
    FlyShadow { target: menuRect }
    Rectangle {
        id: menuRect
        width: 0; height: 0
        //anchors.right: parent.right
        color: styles.colors.white

        ListView {
            id: listView

            readonly property real itemHeight: root.itemHeight

            anchors.fill: parent; anchors.margins: 1
            clip: true
            delegate: Item {
                id: listItem
                width: listView.width; height: listView.itemHeight
                clip: true

                FlyTouchAnimation { mouseArea: listItemMouseArea }

                FlyLabel {
                    anchors { fill: parent; leftMargin: height * 0.5; rightMargin: anchors.leftMargin }
                    text: modelData
                    font.pixelSize: height * 0.4; fontSizeMode: Text.FixedSize;
                    verticalAlignment: Text.AlignVCenter
                    elide: Text.ElideRight
                }

                MouseArea {
                    id: listItemMouseArea
                    anchors.fill: parent
                    onClicked: {
                        contextMenu.state = "";
                        if(contextMenu.clicked) contextMenu.clicked(index);
                    }
                }

                FlySeparator { anchors.bottom: parent.bottom }
            }
        }
    }

    FlyLabel { // HACK
        id: dummyLabel
        height: listView.itemHeight
        text: "XXXXXXXXXXXXXXXXXXXX"
        font.pixelSize: height * 0.4;
        visible: false
    }
}
