import QtQuick 2.3
import QtGraphicalEffects 1.0

Item {
    id: controlButton

    property alias image: image
    property alias rectVisible: imageRect.visible
    property alias text: txt.text
    property alias textColor: txt.color
    property alias fontSize: txt.font.pixelSize
    property string pressAndHoldText: ""

    property bool canBeOff: false
    property bool on: true

    property color lightColor: styles.highlightColor
    property color darkColor: styles.colors.whiteB

    signal clicked
    signal pressAndHold

    width: height; height: parent ? parent.height : 0
    enabled: (opacity === 0 || !visible) ? false : true

    Loader {
        id: tooltipLoader
        active: false
        sourceComponent: Component {
            Item {
                x: controlButton.width / 2 - width / 2
                width: pressAndHoldLabel.paintedWidth + 30 * dpiRatio; height: root.itemHeight * 0.8
                anchors { bottom: parent.top; bottomMargin: 20 * dpiRatio }
                visible: pressAndHoldLabel.text && mouseArea.wasHeld && mouseArea.containsMouse

                RectangularGlow {
                    anchors.fill: tooltipRect
                    glowRadius: 5
                    spread: 0.2
                    color: root.getColorString(0xAA, styles.colors.black)
                    cornerRadius: tooltipRect.radius + glowRadius
                    visible: !lowEffects
                }
                Rectangle {
                    id: tooltipRect;
                    anchors.fill: parent;
                    color: styles.colors.white;
                    radius: 6 * dpiRatio
                    border { width: lowEffects ? 1 : 0; color: styles.colors.gray9 }
                }

                FlyLabel {
                    id: pressAndHoldLabel
                    anchors.centerIn: parent
                    font.pixelSize: parent.height * 0.6
                    text: pressAndHoldText
                }
            }
        }
    }

    Item { id: rectContainer; width: parent.width; height: parent.height * (txt.visible ? 0.7 : 1.0) }
    Rectangle {
        id: imageRect
        anchors.centerIn: rectContainer
        width: Math.min(rectContainer.width, rectContainer.height); height: width
        radius: width / 2
        color: controlButton.on ? lightColor : darkColor
    }

    FlyTouchAnimation {
        isRect: false;
        mouseArea: mouseArea;
        backColor: root.getColorString(0x66, lightColor);
        frontColor: backColor
    }

    FlyColoredImage {
        id: image
        anchors { fill: imageRect; margins: imageRect.width * (imageRect.visible ? 0.2 : 0.1) }
        //imageFillMode: Image.PreserveAspectFit
        imageColorEnabled: canBeOff ? true : false
        imageColor: controlButton.on ? "transparent" : styles.colors.whiteD
        imageSourceSize: Qt.size(imageRect.width, imageRect.height) //Qt.size(width + 5, height + 5)

        Behavior on imageColor { ColorAnimation { } }
    }

    FlyLabel {
        id: txt
        anchors { left: parent.left; right: parent.right; bottom: parent.bottom }
        height: parent.height * 0.3
        text: ""
        font { pixelSize: height * 0.55; bold: true }
        fontSizeMode: Text.FixedSize
        wrapMode: Text.WordWrap
        lineHeight: 0.75
        horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter
        visible: text !== ""
        color: imageRect.color
    }

    MouseArea {
        id: mouseArea

        property bool wasHeld: false

        anchors.fill: parent
        onClicked: controlButton.clicked()
        onPressAndHold: { if(pressAndHoldText && !tooltipLoader.active) tooltipLoader.active = true; wasHeld = true }
        onCanceled: wasHeld = false
        onReleased: {
            wasHeld = false
            if(mouse.wasHeld && containsMouse)
                controlButton.pressAndHold()
        }
    }
}
