import QtQuick 2.3

FocusScope {
    id: listViewScope

    property int currentIndex: -1 // if currentIndex >= 0 then it's radiolist
    property alias list: listView
    signal clicked(int index, string item)

    property var enabledList: [] // [function() { return true }]
    property bool deleteMode: false // if true then model [[data, canBeDeleted], ...]
    signal itemDeleted(int index, string item)

    ListView {
        id: listView
        anchors.fill: parent; anchors.margins: 1
        model: []
        clip: true
        delegate: Item {
            id: listItem

            property bool   canBeDeleted: listViewScope.deleteMode ? modelData[1] : false
            property string itemString: listViewScope.deleteMode ? modelData[0] : modelData

            enabled: index < enabledList.length
                     ? (typeof enabledList[index] === "function"
                        ? enabledList[index]()
                        : enabledList[index])
                     : true
            width: listView.width; height: root.itemHeight
            clip: true

            FlyTouchAnimation { mouseArea: listItemMouseArea }

            Rectangle {
                id: radioRect
                x: 10 * dpiRatio; anchors.verticalCenter: parent.verticalCenter
                width: parent.height * 0.4; height: width
                radius: width / 2
                color: "transparent"
                border { width: 1; color: listViewScope.currentIndex === index ? styles.highlightColor : styles.colors.lightgray }
                visible: listViewScope.currentIndex >= 0

                Rectangle {
                    anchors { fill: parent; margins: parent.width * 0.25 }
                    radius: width / 2
                    color: styles.highlightColor
                    visible: listViewScope.currentIndex === index
                }
            }

            FlyLabel {
                id: listItemLabel
                x: 10 * dpiRatio + (radioRect.visible ? radioRect.x + radioRect.width : 0)
                height: parent.height
                width: Math.min(implicitWidth, parent.width - x - (listItemDeleteButton.visible ? listItemDeleteButton.width : 0))
                pressed: listItemMouseArea.pressed
                text: listItem.itemString.split('\n').join(' ');
                verticalAlignment: Text.AlignVCenter
                fontSizeMode: Text.FixedSize;
                elide: Text.ElideRight
            }

            MouseArea {
                id: listItemMouseArea
                anchors.fill: parent
                onClicked: {
                    if(listViewScope.currentIndex >= 0)
                        listViewScope.currentIndex = index
                    listViewScope.clicked(index, listItem.itemString)
                }
            }

            FlyMenuButton {
                id: listItemDeleteButton;
                anchors.right: parent.right
                height: parent.height
                visible: listItem.canBeDeleted
                text: "\u2717"
                onClicked: {
                    listViewScope.itemDeleted(index, listItem.itemString)
                    var newModel = listView.model
                    newModel.splice(index, 1)
                    listView.model = newModel
                }
            }

            FlySeparator { anchors.bottom: parent.bottom }
        }
    }

    FlyScrollBar {
        anchors.fill: parent
        contentItem: listView
    }
}
