import QtQuick 2.3

Item {
    id: main

    property alias listView: listViewRow
    property var textModel: [] // Title labels
    property var listModel: [] // Components

    Row {
        id: row
        width: parent.width; height: root.itemHeight

        Repeater {
            id: repeater
            model: main.textModel

            FlyLabel {
                width: row.width / repeater.count; height: row.height
                text: String(modelData).toUpperCase()
                font.pixelSize: height * 0.4
                font.bold: true
                horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter

                MouseArea {
                    anchors.fill: parent
                    onClicked: listViewRow.positionViewAtIndex(index, ListView.SnapPosition)
                }

                FlySeparator { horizontal: false; anchors.right: parent.right; visible: index !== repeater.count - 1 }
            }
        }
    }

    Rectangle {
        x: listViewRow.currentIndex >= 0 ? listViewRow.currentIndex * width : 0
        width: row.width / repeater.count - 1; height: row.height * 0.15
        anchors.bottom: row.bottom
        color: styles.highlightColor
        Behavior on x { NumberAnimation { duration: 500; easing.type: Easing.OutBack } }
    }

    Rectangle {
        id: listViewRect
        anchors { left: parent.left; top: row.bottom; right: parent.right; bottom: parent.bottom }
        color: styles.colors.white

        ListView {
            id: listViewRow
            anchors.fill: parent
            model: main.listModel
            clip: true
            orientation: ListView.Horizontal
            snapMode: ListView.SnapOneItem
            boundsBehavior: Flickable.StopAtBounds
            preferredHighlightBegin: 0.5; preferredHighlightEnd: 0.5
            highlightRangeMode: ListView.StrictlyEnforceRange
            cacheBuffer: Math.max(1, width * count)
            delegate: Loader {
                id: listItem
                width: listViewRow.width; height: listViewRow.height
                clip: true
                sourceComponent: modelData
            }
        }
    }
    FlyInnerShadow { target: listViewRect }
}
