import QtQuick 2.3

ListView {
    id: listViewRow

    property color rectColor: styles.highlightColor2
    property int   rectHeight: 10

    property color textColor: styles.label.color
    property int   textSize: styles.label.medium

    clip: true
    orientation: ListView.Horizontal
    leftMargin: width * 0.3; rightMargin: leftMargin

    delegate: FlyLabel {
        id: label
        width: Math.max(contentWidth + 40 * dpiRatio, ListView.view.width * 0.2)
        height: ListView.view.height
        text: modelData.toUpperCase()
        fontSizeMode: Text.FixedSize
        color: textColor
        font { pixelSize: textSize; bold: true }
        horizontalAlignment: Text.AlignHCenter; verticalAlignment: Text.AlignVCenter
        opacity: ListView.isCurrentItem ? 1.0 : 0.7

        FlyTouchAnimation {
            mouseArea: mouseArea;
            backColor: root.getColorString(0x66, rectColor);
            frontColor: backColor
            z: -1
        }

        MouseArea {
            id: mouseArea
            anchors.fill: parent
            onClicked: label.ListView.view.currentIndex = index
        }
    }

    highlightFollowsCurrentItem: false
    highlight: Rectangle {
        x: ListView.view.currentItem ? ListView.view.currentItem.x : 0;
        width: ListView.view.currentItem ? ListView.view.currentItem.width : 0;
        anchors.bottom: parent.bottom; height: rectHeight
        color: rectColor

        Behavior on x { NumberAnimation { } }
        Behavior on width { NumberAnimation { } }
    }

    function fixContentX() {
        if(currentItem)
            contentX = Math.min(Math.max(currentItem.x - width / 2 + currentItem.width / 2, -leftMargin), contentWidth + rightMargin - width)
    }

    onCurrentItemChanged: fixContentX()
    onWidthChanged: fixContentXDelayTimer.restart()

    Timer { id: fixContentXDelayTimer; interval: 100; onTriggered: fixContentX() }

    Behavior on contentX { enabled: !listViewRow.moving; NumberAnimation { } }
}
