import QtQuick 2.3

Rectangle {
    id: button

    property alias image: image
    property alias text: txt.text
    property alias textItem: txt
    property alias textColor: txt.color
    property alias fontSize: txt.font.pixelSize
    property alias comboBoxDecor: comboBoxDecor
    property alias buttonRow: buttonRow
    property bool  checked: false
    property real  margin: 10 * dpiRatio
    property real  imageMarginRatio: 0.7
    property color pressColor: root.getColorString(0x66, Qt.tint(styles.highlightColor, "#40000000"))

    signal clicked
    signal pressAndHold

    width: buttonRow.width + margin * 2; height: parent.height
    color: "transparent"
    enabled: (opacity === 0 || !visible) ? false : true
    clip: true

    FlyTouchAnimation {
        mouseArea: mouseArea;
        backColor: pressColor
        frontColor: pressColor
        anchors.margins: -parent.width * 0.2
    }

    Rectangle { anchors.fill: parent; color: pressColor; visible: checked }

    Row {
        id: buttonRow
        x: margin
        height: parent.height
        spacing: button.margin

        FlyColoredImage {
            id: image
            anchors.verticalCenter: parent.verticalCenter
            width: height; height: parent.height * button.imageMarginRatio
            imageSourceSize: Qt.size(button.height, button.height)
        }

        FlyLabel {
            id: txt
            anchors.verticalCenter: parent.verticalCenter
            text: ""
            font.pixelSize: parent.height * 0.5
            transformOrigin: Item.Bottom
            //scale: mouseArea.pressed || button.checked ? 1.05 : 1.0
            visible: text !== ""
        }
    }

    Rectangle {
        id: comboBoxDecor
        anchors { bottom: parent.bottom }
        width: parent.width; height: 1
        color: styles.label.colorDisabled
        visible: false

        Rectangle {
            anchors { horizontalCenter: parent.right; verticalCenter: parent.bottom }
            width: 10 * dpiRatio; height: width
            color: parent.color
            rotation: 45
        }
    }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        onClicked: button.clicked()
        onPressAndHold: button.pressAndHold()
    }
}
