import QtQuick 2.3

MouseArea {
    id: selectMouseArea

    readonly property var  regexIsNotSpace: /\S/
    property var selection: root.selection
    property var inputTarget: null
    property var flickable: null

    anchors.fill: inputTarget
    enabled: !isDesktopMode

    function showSelection(mouse) {
        if(!inputTarget) return;

        if(!inputTarget.activeFocus) { inputTarget.forceActiveFocus(Qt.MouseFocusReason); return; }

        selection.textChangedByOutside = false
        if(Qt.inputMethod) Qt.inputMethod.commit()

        var pos = inputTarget.positionAt(mouse.x, mouse.y);
        inputTarget.select(pos, pos + 1); // HACK for vkbd prediction
        inputTarget.select(pos, pos);
        var posChar = inputTarget.getText(pos, pos + 1)
        if(regexIsNotSpace.test(posChar))
            inputTarget.selectWord();
        selection.textChangedByOutside = true

        selection.show(inputTarget, flickable, true)
    }

    function showAndInitInputMethod() {
        if(Qt.inputMethod && !Qt.inputMethod.visible) {
            Qt.inputMethod.show()
            if(root.virtualKeyboard)
                root.virtualKeyboard.setActiveItem(inputTarget, !!flickable)
        }
    }

    onClicked: {
        if(!inputTarget) return;
        if(selection.inputTarget && selection.selectionVisible) { selection.done(); return; }

        if(!inputTarget.activeFocus) { inputTarget.forceActiveFocus(Qt.MouseFocusReason); return; }

        if(!inputTarget.length) { showAndInitInputMethod(); return; }

        selection.textChangedByOutside = false
        if(Qt.inputMethod) Qt.inputMethod.commit()

        var pos = inputTarget.positionAt(mouse.x, mouse.y);
        inputTarget.select(pos, pos + 1); // HACK for vkbd prediction
        inputTarget.select(pos, pos);
        selection.textChangedByOutside = true

        selection.show(inputTarget, flickable, false)

        showAndInitInputMethod()
    }

    onPressAndHold: showSelection(mouse);
    onDoubleClicked: showSelection(mouse);
}
