import QtQuick 2.3

FlyNotificationItem {
    id: notificationItem

    readonly property bool isIncomingCall: _notification && _notification.dataMap["origState"] === "incoming"
    property string stateText: ""

    state: _notification && _notification.dataMap["state"] ? _notification.dataMap["state"] : ""
    icon.imagePath: "image://ContactImageProvider/"
                    + (_notification && _notification.dataMap["contactId"] ? _notification.dataMap["contactId"] : "")
    caption.width: datetime.x - caption.x - 5
    caption.text: _notification && _notification.dataMap["contactFullname"]
                  ? _notification.dataMap["contactFullname"]
                  : _notification.dataMap["phone"]
    description.text: stateText + ". " + (_notification && _notification.dataMap["contactFullname"] ? _notification.dataMap["phone"] : "")

    states: [
        State {
            name: "incoming"
            PropertyChanges { target: notificationItem; stateText: qsTr("Incoming call") }
        },
        State {
            name: "outgoing"
            PropertyChanges { target: notificationItem; stateText: qsTr("Outgoing call") }
        },
        State {
            name: "missed"
            PropertyChanges { target: notificationItem; stateText: qsTr("Missed call") }
        },
        State {
            name: "online"
            PropertyChanges { target: notificationItem; stateText: qsTr("Online") }
        }
    ]

    function clicked() {
        if(state === "missed") {
            root.startDetachedProcess("fly-qml-dialer", ["--call", _notification.dataMap["phone"]]);
        } else {
            dialerWindow.appear() // TODO HACK
        }


        // TODO if missed then call

        return false; // close notification panel
    }

    FlyLabel {
        id: datetime
        anchors { top: parent.top; right: parent.right; bottom: parent.verticalCenter; rightMargin: margin }
        text: notificationItem.state === "online"
              ? dialerWindow.innerPage.durationText // TODO HACK
              : Qt.formatDateTime((_notification && _notification.dataMap["datetime"]
                                   ? new Date(_notification.dataMap["datetime"])
                                   : new Date()), "hh:mm")
        font.pixelSize: description.font.pixelSize;
        color: styles.label.colorDisabled
        verticalAlignment: Text.AlignVCenter
    }

    FlyColoredImage {
        id: typeImage
        anchors { top: parent.verticalCenter; right: parent.right; bottom: parent.bottom;
            topMargin: margin; rightMargin: margin; bottomMargin: margin }
        width: height
        imagePath: isIncomingCall ? "qrc:/images/arrowback.png" : "qrc:/images/arrowforward.png"
        imageColorEnabled: true
        imageColor: notificationItem.state === "missed"
                    ? root.getColorString(0xAA, styles.colors.red)
                    : isIncomingCall
                      ? root.getColorString(0xAA, styles.colors.blueLight)
                      : root.getColorString(0x80, styles.colors.green)
        rotation: -45
    }
}
