import QtQuick 2.3

Item {
    id: notificationItem
    height: root.itemHeight * 2

    readonly property alias icon: icon
    readonly property alias caption: caption
    readonly property alias description: description
    property real margin: parent.height * 0.1

    function clicked() {
        if("click" in _notification.dataMap) {
            if(_notification.dataMap["click"]) {
                var args = _notification.dataMap["click"].split(" ");
                root.startDetachedProcess(args.shift(), args);
            }
        } else {
            root.showMessagePage({
                                     "title": caption.text,
                                     "message": description.text,
                                     "titleImage.imagePath": icon.imagePath,
                                     "showCancelButton": false
                                 })
        }

        return false; // close notification panel
    }

    FlyColoredImage {
        id: icon
        x: margin; y: margin
        width: parent.height * 0.8; height: width
        imageFillMode: Image.PreserveAspectFit
        imagePath: _notification && _notification.dataMap["icon"] ? _notification.dataMap["icon"] : "image://ThemeImageProvider/task-attention"
    }

    FlyLabel {
        id: caption
        x: icon.x + icon.width + margin; y: 0
        width: parent.width - x - margin; height: parent.height * 0.5
        text: _notification && _notification.dataMap["caption"] ? _notification.dataMap["caption"] : ""
        font.pixelSize: height * 0.55; fontSizeMode: Text.FixedSize;
        verticalAlignment: Text.AlignBottom
        elide: Text.ElideRight
    }

    FlyLabel {
        id: description
        x: icon.x + icon.width + margin; y: parent.height * 0.5
        width: parent.width - x - margin; height: parent.height * 0.5
        text: _notification && _notification.dataMap["description"] ? _notification.dataMap["description"] : caption.text
        font.pixelSize: height * 0.4; fontSizeMode: Text.FixedSize;
        color: styles.label.colorDisabled
        verticalAlignment: Text.AlignTop
        elide: Text.ElideRight
    }
}
