import QtQuick 2.3
import QtGraphicalEffects 1.0

FocusScope {
    id: page

    // TODO need to fully refactor this module

    property alias title: titleText.text
    property alias titleRect: titleRect
    property alias titleLabel: titleText
    property alias closeButton: closeButton
    property alias frontRect: pageRect
    property alias backRect: backRect
    property alias titleImage: image
    property alias color: pageRect.color

    property alias okButtonText: okButton.buttonText
    property alias okButtonEnabled: okButton.enabled
    property bool  showOkButton: true
    property alias cancelButtonText: cancelButton.buttonText
    property bool  showCancelButton: true

    property var   okFunctionCustomArgs: []
    property var   okFunction
    property var   cancelFunction

    property alias showAnimation: showAnimation
    property alias hideAnimation: hideAnimation

    property alias centerRect: centerRect
    property alias centerRectMargin: centerRect.margin
    property alias centerRectShadow: centerRectShadow.visible

    property real  leftMargin: parent ? parent.width * root.pageMargin : 0
    property real  topMargin: parent ? parent.height * root.pageMargin : 0

    property int   pageIndex: 0

    property alias bottomLeftSpace: bottomLeftSpace
    property alias bottomRightSpace: bottomRightSpace

    width: parent.width
    height: root.virtualKeyboard && Qt.inputMethod.visible && root.currentPage === page ? root.virtualKeyboard.y : parent.height
    focus: true
    enabled: opacity === 1.0

    function closePage(ok) { // .apply is used to call endFunction with custom arguments
        var customArgs = []
        if(arguments.length > 1)
            customArgs = Array.prototype.slice.call(arguments, 1);

        var boundRemovePage = root.removePage.bind(this) // in case okFunction or cancelFunction changes current pageStack

        if(Qt.inputMethod) Qt.inputMethod.commit()

        var dontClosePage = false

        if(ok) {
            if(okFunction && typeof okFunction === "function") dontClosePage = okFunction.apply(this, customArgs)
        } else {
            if(cancelFunction && typeof cancelFunction === "function") dontClosePage = cancelFunction.apply(this, customArgs)
        }

        if(!dontClosePage) boundRemovePage(pageIndex)
    }

    MouseArea { anchors.fill: parent }
    Rectangle { id: backRect; anchors.fill: parent; color: styles.page.backgroundColor }

    RectangularGlow {
        anchors.fill: pageRect
        glowRadius: 10
        spread: 0.2
        color: root.getColorString(0x44, styles.colors.black)
        cornerRadius: pageRect.radius + glowRadius
        visible: pageRect.visible && (pageRect.width < parent.width * 0.9 || pageRect.height < parent.height * 0.9) && !lowEffects
    }
    Rectangle { anchors { fill: pageRect; margins: -1 } visible: pageRect.visible && (pageRect.width < parent.width * 0.9 || pageRect.height < parent.height * 0.9) && lowEffects; color: styles.colors.gray9; radius: pageRect.radius }
    Rectangle {
        id: pageRect
        anchors { fill: parent; leftMargin: page.leftMargin; topMargin: page.topMargin;
            rightMargin: anchors.leftMargin; bottomMargin: anchors.topMargin }
        border.width: styles.page.borderWidth; border.color: styles.page.borderColor; radius: styles.page.radius
        color: styles.page.foregroundColor
        enabled: visible

        FlyShadow { target: titleRect; horizontalOffset: 0; verticalOffset: 1 * dpiRatio }
        Rectangle {
            id: titleRect
            x: 1; z: 1
            width: parent.width - 2; height: root.itemHeight
            color: styles.page.titleColor

            FlyColoredImage {
                id: image
                anchors { left: parent.left; leftMargin: parent.height * 0.2; verticalCenter: parent.verticalCenter }
                imageSourceSize: Qt.size(parent.height, parent.height)
                width: height; height: parent.height * 0.7
            }

            FlyLabel {
                id: titleText
                anchors { left: image.right; top: parent.top; right: closeButton.left; bottom: parent.bottom;
                    leftMargin: height * 0.2 }
                text: ""
                color: styles.colors.white
                font.pixelSize: styles.label.big;
                //font.bold: true
                verticalAlignment: Text.AlignVCenter
                fontSizeMode: Text.FixedSize;
                elide: Text.ElideRight
            }

            FlyCloseButton {
                id: closeButton
                anchors.right: parent.right
                onClicked: page.closePage(false)
            }
        }

        FlyButton {
            id: okButton
            anchors { bottom: parent.bottom; bottomMargin: centerRect.margin;
                horizontalCenter: cancelButton.visible ? undefined : parent.horizontalCenter;
                right: cancelButton.visible ? parent.horizontalCenter : undefined; rightMargin: 5 }
            width: Math.min(parent.width * 0.4, 200 * dpiRatio); height: root.itemHeight * 0.8
            colorLight: styles.highlightColor2
            colorDark: root.getColorString(0x66, styles.highlightColor)
            boldText: true
            buttonText: qsTr("OK")
            visible: page.showOkButton
            onClicked: page.closePage.apply(this, [true].concat(page.okFunctionCustomArgs))
        }

        FlyButton {
            id: cancelButton
            anchors { bottom: parent.bottom; bottomMargin: centerRect.margin;
                horizontalCenter: okButton.visible ? undefined : parent.horizontalCenter;
                left: okButton.visible ? parent.horizontalCenter : undefined; leftMargin: 5 }
            width: Math.min(parent.width * 0.4, 200 * dpiRatio); height: root.itemHeight * 0.8
            buttonText: qsTr("Close")
            visible: page.showCancelButton
            onClicked: page.closePage(false)
        }

        Item {
            id: bottomLeftSpace
            anchors { left: parent.left; right: okButton.visible ? okButton.left : cancelButton.left; bottom: parent.bottom;
                margins: centerRect.margin }
            height: okButton.height
        }

        Item {
            id: bottomRightSpace
            anchors { left: cancelButton.visible ? cancelButton.right : okButton.right; right: parent.right; bottom: parent.bottom;
                margins: centerRect.margin }
            height: okButton.height
        }
    }

    Rectangle {
        id: centerRect

        property real margin: 10 * dpiRatio

        anchors { left: pageRect.left; right: pageRect.right; margins: centerRect.margin }
        y: pageRect.y + titleRect.height + centerRect.margin
        height: pageRect.y + (showOkButton || showCancelButton ? okButton.y : pageRect.height) - y - anchors.bottomMargin
        color: centerRectShadow.visible ? styles.page.centerRectColor : "transparent"
    }
    FlyInnerShadow { id: centerRectShadow; target: centerRect; visible: false; enabled: visible; z: 999 }

    // @
    NumberAnimation on opacity { id: showAnimation; from: 0; to: 1 }
    NumberAnimation on opacity { id: hideAnimation; running: false; from: 1; to: 0.1; onStopped: page.destroy() }
}
