import QtQuick 2.3
import QtGraphicalEffects 1.0
import Fly.Components 1.0

Item {
    id: pageStack

    property FlyPage initialPage
    property var currentItem: undefined
    property var componentsCache: [["initialPage", initialPage]]
    property var itemArray: []
    property var backStackFunctions: []
    property bool isMain: true

    function push(pagePath, pageProp) {
        if(!pageProp)
            pageProp = {}

        var component
        for(var i = 0; i < componentsCache.length; i++) {
            if(componentsCache[i][0] === pagePath) {
                component = componentsCache[i][1]
                break;
            }
        }

        if(!component) {
            component = Qt.createComponent(pagePath);

            if(component.status == Component.Error) {
                console.log("Error loading component", pagePath, component.errorString())
                return null;
            }

            componentsCache.push([pagePath, component]) // TODO remove unused components from cache on some reason
        }

        if(itemArray.length > 0 && currentItem)
            currentItem.parent = blurRect

        var item = component.createObject(pageStack, pageProp);
        if(item == null) {
            console.log("Error creating object", pagePath);
            return null;
        }

        itemArray.push(item)
        currentItem = item

        item.pageIndex = itemArray.length - 1
        item.focus = true

        if(itemArray.length === 1 && isMain) { // for initial page
            item.showAnimation.stop()
            item.opacity = 1
        } else if(itemArray.length > 1) {
            addFunctionToBackStack(defaultBackAction)
        }

        return item;
    }

    function pop(index) {
        if(itemArray.length === 1) {
            if(isMain)
                console.log("Error! You are trying to close initial page!")
            return
        }

        if(!index)
            index = itemArray.length - 1

        if(index >= 0 && index < itemArray.length) {
            itemArray[index].z = 1
            itemArray[index].hideAnimation.start()
            itemArray.splice(index, 1)
            popFunctionFromBackStack()

            for(var i = index; i < itemArray.length; i++) {
                itemArray[i].pageIndex -= 1
            }

            currentItem = itemArray[itemArray.length - 1]
            currentItem.parent = pageStack
            currentItem.focus = true
        } else {
            console.log("Error! You are trying to remove page with unknown index!", index)
        }
    }

    function defaultBackAction() {
        if(currentItem && currentItem.closePage) {
            try {
                currentItem.closePage(false)
            } catch(e) {
                console.warn(e)
                pop()
            }
        } else {
            pop()
        }
    }

    function processBackClick() {
        if(backStackFunctions.length > 0) {
            var f = backStackFunctions[backStackFunctions.length - 1];

            if(f && typeof f === "function") {
                try {
                    var result = f();

                    if(result && !isNaN(result) && result < 0) {
                        popFunctionFromBackStack()
                        processBackClick()
                    }
                } catch(e) {
                    console.warn(e)
                    popFunctionFromBackStack()
                    processBackClick()
                }
            } else {
                popFunctionFromBackStack()
            }
        } else {
            updateBackStackEmptyFlag()
        }
    }

    function updateBackStackEmptyFlag() {
        if("backStackIsEmpty" in parent) {
            parent.backStackIsEmpty = backStackFunctions.length === 0
        } else {
            backStackIsEmpty = backStackFunctions.length === 0
        }
    }

    function addFunctionToBackStack(f) {
        if(isDesktopMode) return

        backStackFunctions.push(f)
        updateBackStackEmptyFlag()
    }

    function popFunctionFromBackStack() {
        backStackFunctions.pop()
        updateBackStackEmptyFlag()
    }

    function clearBackStack() {
        backStackFunctions = []
        updateBackStackEmptyFlag()
    }

    Item { id: blurRect; anchors.fill: parent }
    FastBlur { id: blur; anchors.fill: parent; source: blurRect; radius: 16 }
}
