import QtQuick 2.3

Grid {
    id: keysGrid
    rows: 4; columns: 3
    enabled: visible

    signal clicked(string text)
    signal pressAndHold(string text)

    Repeater {
        id: keysRepeater

        readonly property var buttonsData: [
            ["1", "", ""],
            ["2", "ABC", "АБВГ"],
            ["3", "DEF", "ДЕЁЖЗ"],
            ["4", "GHI", "ИЙКЛ"],
            ["5", "JKL", "МНОП"],
            ["6", "MNO", "РСТУ"],
            ["7", "PQRS", "ФХЦЧ"],
            ["8", "TUV", "ШЩЪЫ"],
            ["9", "WXYZ", "ЬЭЮЯ"],
            ["*", "", ""],
            ["0", "+", ""],
            ["#", "", ""]
        ]

        width: keysGrid.width; height: keysGrid.height
        model: keysGrid.rows * keysGrid.columns

        Rectangle {
            id: keyRect

            readonly property bool onlyEn: (keysRepeater.buttonsData[index][2] === "") || (root.locale.name !== "ru_RU")
            readonly property real minSize: Math.min(width, height)

            width: keysGrid.width / keysGrid.columns; height: keysGrid.height / keysGrid.rows
            color: keyMouseArea.pressed ? styles.colors.black1 : "transparent"

            Text {
                id: keyNumber
                x: parent.width / 2 - width / ((keysRepeater.buttonsData[index][1] === "") ? 2 : 1)
                anchors.verticalCenter: parent.verticalCenter
                text: keysRepeater.buttonsData[index][0]
                color: styles.highlightColor2
                font.pixelSize: keyRect.minSize * 0.6
                style: Text.Raised; styleColor: Qt.darker(styles.highlightColor2)
                renderType: Text.NativeRendering
            }

            Text {
                anchors { left: keyNumber.right; bottom: parent.verticalCenter; leftMargin: 10 * dpiRatio }
                height: keyRect.minSize * 0.2; width: parent.width - x
                text: keyRect.onlyEn ? "" : keysRepeater.buttonsData[index][2]
                color: styles.highlightColor2
                font.pixelSize: height; fontSizeMode: Text.HorizontalFit
                minimumPixelSize: height * 0.3
                style: Text.Raised; styleColor: Qt.darker(styles.highlightColor2)
                renderType: Text.NativeRendering
            }

            Text {
                id: keyExt
                anchors { left: keyNumber.right; top: keyRect.onlyEn ? undefined : parent.verticalCenter;
                    verticalCenter: keyRect.onlyEn ? parent.verticalCenter : undefined;
                    leftMargin: 10 * dpiRatio }
                height: keyRect.minSize * 0.2; width: parent.width - x
                text: keysRepeater.buttonsData[index][1]
                color: keyRect.onlyEn ? styles.highlightColor2 : Qt.darker(styles.highlightColor2)
                font.pixelSize: height; fontSizeMode: Text.HorizontalFit
                minimumPixelSize: height * 0.3
                style: keyRect.onlyEn ? Text.Raised : Text.Normal; styleColor: Qt.darker(styles.highlightColor2)
                renderType: Text.NativeRendering
            }

            Rectangle { anchors.right: parent.right; width: 1; height: parent.height; color: styles.colors.black4; visible: (parent.Positioner.index + 1) % keysGrid.columns !== 0 }
            Rectangle { anchors.bottom: parent.bottom; width: parent.width; height: 1; color: styles.colors.black4; visible: parent.Positioner.index < keysRepeater.count - keysGrid.columns }

            MouseArea {
                id: keyMouseArea
                anchors.fill: parent
                onClicked: keysGrid.clicked(keyNumber.text)
                onPressAndHold: keysGrid.pressAndHold((keyExt.text === "+" ? keyExt.text : keyNumber.text))
            }

            Behavior on color { ColorAnimation {} }
        }
    }
}
