import QtQuick 2.3

Item {
    id: radioButtonRow

    property int currentIndex: -1
    property alias model: radioButtonRepeater.model
    signal clicked(int index)

    Binding {
        target: radioButtonRow
        when: handleMouseArea.pressed
        property: "currentIndex"
        value: Math.round(handle.x / handle.width)
    }

    Rectangle {
        id: rowBackground
        anchors { fill: parent; margins: parent.height * 0.1 }
        color: styles.colors.white
    }

    FlyInnerShadow { target: rowBackground }
    FlyShadow { target: handle; horizontalOffset: -1; verticalOffset: 1 }

    Rectangle {
        id: handle
        width: buttonRow.width / radioButtonRepeater.count; height: parent.height
        x: radioButtonRow.currentIndex * width
        color: styles.highlightColor2

        Behavior on x { NumberAnimation {} }
    }

    Row {
        id: buttonRow
        anchors.fill: parent

        Repeater {
            id: radioButtonRepeater
            width: buttonRow.width; height: buttonRow.height
            model: []
            delegate: Item {
                id: radioButtonItem
                height: buttonRow.height
                width: buttonRow.width / radioButtonRepeater.count

                FlyLabel {
                    anchors { fill: parent; leftMargin: 2; rightMargin: 2 }
                    pressed: itemMouseArea.pressed
                    text: modelData
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                    elide: Text.ElideRight
                    enabled: radioButtonRow.currentIndex === index
                }

                MouseArea {
                    id: itemMouseArea
                    anchors.fill: parent
                    onClicked: {
                        if(radioButtonRow.currentIndex !== index) {
                            radioButtonRow.currentIndex = index
                            handle.x = Qt.binding(function() {return radioButtonRow.currentIndex * handle.width})
                            radioButtonRow.clicked(index)
                        }
                    }
                }
            }
        }
    }

    MouseArea {
        id: handleMouseArea
        anchors.fill: handle
        enabled: radioButtonRepeater.count > 1
        drag {
            target: handle
            axis: Drag.XAxis
            minimumX: 0
            maximumX: handle.width * (radioButtonRepeater.count - 1)
        }
        onReleased: handle.x = Math.round(handle.x / handle.width) * handle.width
    }
}
