import QtQuick 2.3

Item {
    id: settingsItem
    width: parent.ListView.view.width; height: contentItem.height
    enabled: itemData.enabled
    clip: true // for touch animation

    property bool clickFuncOverride: false
    readonly property alias contentItem: contentItem
    readonly property alias mouseArea: settingsItemMouseArea

    FlyTouchAnimation { mouseArea: settingsItemMouseArea }

    Item {
        id: contentItem
        width: parent.width; height: Math.max(childrenRect.height + (description.lineCount > 1 ? 6 * dpiRatio : 0), itemData.height)

        FlyColoredImage {
            id: icon
            x: itemData.icon ? 10 * dpiRatio : -width
            anchors.verticalCenter: parent.verticalCenter
            width: height; height: itemData.height * 0.6
            imagePath: itemData.icon
            imageColorEnabled: !!itemData.iconColor
            imageColor: itemData.iconColor ? itemData.iconColor : styles.colors.white
        }

        FlyLabel {
            id: name
            x: icon.x + icon.width + 10 * dpiRatio; y: description.text === "" ? itemData.height * 0.15 : 0
            width: parent.width - x - 10 * dpiRatio; height: itemData.height * 0.5
            text: itemData.name
            font.pixelSize: height * 0.55; fontSizeMode: Text.FixedSize;
            verticalAlignment: Text.AlignBottom
            elide: Text.ElideRight
        }

        FlyLabel {
            id: description
            y: itemData.height * 0.5 + 3 * dpiRatio;// height: parent.height - y - 2
            anchors { left: icon.right; right: parent.right; leftMargin: 10 * dpiRatio; rightMargin: 10 * dpiRatio }
            text: !itemData.enabled && itemData.disableDescription ? itemData.disableDescription : itemData.description
            font.pixelSize: 17 * dpiRatio; fontSizeMode: Text.FixedSize;
            color: styles.label.colorDisabled
            wrapMode: Text.WordWrap
        }
    }

    MouseArea { id: settingsItemMouseArea; anchors.fill: parent;
        onClicked: if(!settingsItem.clickFuncOverride && itemData.clickFunc) itemData.clickFunc() }

    Rectangle { width: parent.width; height: 1; anchors.bottom: parent.bottom; color: styles.separator.colorDark }
}
