import QtQuick 2.3
import Fly.Components 1.0

ListView {
    id: listView

    //property int deepLevel: 0

    property var sublistComponent: null
    property var sublistObject: null

    property var updatePageTitleFunc: null

    anchors.fill: parent
    model: []
    clip: true
    focus: true
    delegate: Component {
        Loader {
            readonly property var itemData: modelData

            source: switch(itemData.type) {
                    case SettingsItemType.None:
                        return Qt.resolvedUrl("FlySettingsItem.qml");
                    case SettingsItemType.Check:
                        return Qt.resolvedUrl("FlySettingsCheckBoxItem.qml");
                    case SettingsItemType.Number:
                        return Qt.resolvedUrl("FlySettingsSpinBoxItem.qml");
                    case SettingsItemType.String:
                        return Qt.resolvedUrl("FlySettingsLineEditItem.qml");
                    case SettingsItemType.Date:
                        return Qt.resolvedUrl("FlySettingsDateItem.qml");
                    case SettingsItemType.Color:
                        return Qt.resolvedUrl("FlySettingsColorItem.qml");
                    case SettingsItemType.Combo:
                        return Qt.resolvedUrl("FlySettingsComboBoxItem.qml");
                    case SettingsItemType.List:
                        return Qt.resolvedUrl("FlySettingsListViewItem.qml");
                    case SettingsItemType.Category:
                        return Qt.resolvedUrl("FlySettingsCategoryItem.qml");
                    case SettingsItemType.Custom:
                        return itemData.path;
                    default:
                        return "";
                    }

        }
    }

    // creation context here, so destroyed delegate won't destroy sublist
    function openListView(itemData) {
        var component = sublistComponent ? sublistComponent : Qt.createComponent(Qt.resolvedUrl("FlySettingsSubListView.qml"));
        sublistObject = component.createObject(listView.parent,
                                               {
                                                   "x": listView.x,
                                                   "y": listView.y,
                                                   "width": listView.width,
                                                   "height": listView.height,
                                                   "closeListViewFunc": closeListView,
                                                   "updatePageTitleFunc": updatePageTitleFunc,
                                                   "sublistComponent": sublistComponent,
                                                   "itemData": itemData
                                               });

        root.addFunctionToBackStack(closeListView);
    }

    function closeListView() {
        if(sublistObject) {
            sublistObject.closeSubListView();
            sublistObject = null;

            root.popFunctionFromBackStack();
        }
    }

    onSublistObjectChanged: if(updatePageTitleFunc) updatePageTitleFunc();
}
