import QtQuick 2.3

Item {
    id: listViewItem

    readonly property alias sublistObject: subListView.sublistObject

    property alias closeAnimation: closeAnimation

    property var itemData: null
    property var sublistComponent: null
    property var closeListViewFunc: null
    property var updatePageTitleFunc: null

    function closeSubListView() {
        subListView.closeListView();
        closeAnimation.start();
    }

    Rectangle {
        id: listViewRect
        anchors.fill: parent
        color: styles.page.centerRectColor

        FlySettingsItemData {
            id: itemBack;
            name: qsTr("..back");
            description: qsTr("Return back to ") + itemData.returnBackTo
            clickFunc: closeListViewFunc
        }

        FlySettingsListView {
            id: subListView
            anchors.fill: parent
            model: [itemBack].concat(itemData.model)
            sublistComponent: listViewItem.sublistComponent
            updatePageTitleFunc: listViewItem.updatePageTitleFunc
            //deepLevel: settingsItem.parent.ListView.view.deepLevel + 1
            currentIndex: 0
        }
    }

    ParallelAnimation {
        running: true

        NumberAnimation { target: listViewItem; property: "x"; from: listViewItem.width * 0.25; to: x; duration: 250; easing.type: Easing.InOutQuad }
        NumberAnimation { target: listViewItem; property: "opacity"; from: 0; to: 1; duration: 250; easing.type: Easing.InOutQuad }
    }

    SequentialAnimation {
        id: closeAnimation

        ParallelAnimation {
            NumberAnimation { target: listViewItem; property: "x"; to: listViewItem.width * 0.25; duration: 250; easing.type: Easing.InOutQuad }
            NumberAnimation { target: listViewItem; property: "opacity"; to: 0; duration: 250; easing.type: Easing.InOutQuad }
        }

        ScriptAction { script: (function() { listViewItem.destroy(); })() }
    }
}
