import QtQuick 2.3

Item {
    id: slider

    property int index: 0
    property int count: 100
    property int backgroundCount: 0
    property bool tickmarks: false
    readonly property alias mousePressed: handleMouseArea.pressed

    signal indexChangedByUser(int index)

    function increment() {
        if(index < count - 1) index++
    }

    function decrement() {
        if(index > 0) index--
    }

    Rectangle {
        id: line
        anchors.centerIn: parent
        width: parent.width - handle.width; height: Math.max(4, handle.height / 3)
        color: styles.colors.white
        border { color: styles.colors.whiteB; width: 1 }
        radius: height * 0.5

        MouseArea {
            anchors { left: parent.left; right: parent.right; verticalCenter: parent.verticalCenter }
            height: slider.height
            onClicked: {
                xBehavior.enabled = false
                handle.x = mouse.x
                xBehavior.enabled = true
                index = Math.round(handle.x / handle.valueWidth)
                indexChangedByUser(index)
            }
        }
    }

    Rectangle {
        id: lineHighlightRectDouble
        anchors { left: line.left; top: line.top; bottom: line.bottom }
        width: line.width / count * backgroundCount
        color: styles.highlightColor2
        visible: width > 0
        radius: line.radius
    }

    Rectangle {
        id: lineHighlightRect
        anchors { left: line.left; top: line.top; right: handle.right; bottom: line.bottom }
        color: styles.highlightColor
        radius: line.radius
    }

    Repeater { // tickmarks
        anchors.fill: parent
        model: tickmarks ? Math.max(0, slider.count - 2) : 0

        Rectangle {
            anchors.verticalCenter: parent.verticalCenter
            x: line.x + line.width / (slider.count - 1) * (index + 1)
            width: 1; height: line.height
            color: line.border.color
        }
    }

    Rectangle {
        id: handle

        readonly property real valueWidth: line.width / (count - 1)

        anchors.verticalCenter: parent.verticalCenter
        width: Math.min(parent.height, root.itemHeight * 0.5); height: width
        color: handleMouseArea.pressed ? border.color : styles.colors.white
        border { color: lineHighlightRect.color; width: 1 }
        radius: width

        Binding {
            target: slider
            property: "index"
            when: handleMouseArea.drag.active
            value: Math.round(handle.x / handle.valueWidth)
        }

        Binding {
            target: handle
            property: "x"
            when: !handleMouseArea.drag.active
            value: slider.index * handle.valueWidth
        }

        Rectangle {
            id: handleHighlightRect
            anchors.centerIn: parent
            width: parent.width / 3; height: width
            radius: width
            color: lineHighlightRect.color
        }

        MouseArea {
            id: handleMouseArea
            width: height; height: slider.height
            anchors.centerIn: parent
            drag.target: parent
            drag.axis: Drag.XAxis
            drag.minimumX: 0; drag.maximumX: line.width
        }

        Behavior on x { id: xBehavior; NumberAnimation { easing.type: Easing.InOutQuad } }
    }

    onIndexChanged: if(handleMouseArea.drag.active) indexChangedByUser(index)
}
