import QtQuick 2.3

Rectangle {
    id: flySpinBox

    property int minValue: 0
    property int maxValue: 0
    readonly property int currentValue: pathView.currentIndex + minValue
    property alias flickDeceleration: pathView.flickDeceleration
    property alias grooveVisible: groove.visible
    property color mainColor: styles.label.color

    property string prefix: ""
    property string suffix: ""

    color: "transparent"

    function init(curVal) {
        pathView.positionViewAtIndex(curVal - minValue, PathView.SnapPosition)
        handle.y = pathView.currentIndex / pathView.count * (scrollBar.height - handle.width)
        pathView.positionViewAtIndex(curVal - minValue, PathView.SnapPosition) // sometimes it doesn't set right index
    }

    PathView {
        id: pathView
        anchors.fill: parent
        model: flySpinBox.maxValue - flySpinBox.minValue
        pathItemCount: height / root.itemHeight
        delegate: Text {
            opacity: PathView.itemOpacity; scale: PathView.itemScale
            font.pixelSize: height * 0.5
            text: flySpinBox.prefix + (flySpinBox.minValue + index) + flySpinBox.suffix
            color: index === pathView.currentIndex ? styles.highlightColor : flySpinBox.mainColor
            height: root.itemHeight; verticalAlignment: Text.AlignVCenter }
        preferredHighlightBegin: 0.5; preferredHighlightEnd: 0.5
        dragMargin: width
        flickDeceleration: 200;
        path: Path {
            startX: pathView.width/2; startY: 0
            PathAttribute { name: "itemOpacity"; value: 0.1 }
            PathAttribute { name: "itemScale"; value: 0.7 }
            PathLine { x: pathView.width/2; y: pathView.height/2 }
            PathAttribute { name: "itemOpacity"; value: 1.0 }
            PathAttribute { name: "itemScale"; value: 1.3 }
            PathLine { x: pathView.width/2; y: pathView.height }
            PathAttribute { name: "itemOpacity"; value: 0.1 }
            PathAttribute { name: "itemScale"; value: 0.7 }
        }
        Keys.onDownPressed: incrementCurrentIndex()
        Keys.onUpPressed: decrementCurrentIndex()
        onCurrentIndexChanged: {
            if(pathView.moving)
                handle.y = currentIndex / count * (scrollBar.height - handle.width)
        }
    }

    Item {
        id: scrollBar
        anchors { top: parent.top; right: parent.right; bottom: parent.bottom }
        width: parent.width * 0.05
        visible: true; enabled: visible

        MouseArea {
            anchors { left: parent.left; top: groove.top; right: parent.right; bottom: groove.bottom }
            onClicked: {
                var moving = pathView.moving
                if(moving)
                    pathView.positionViewAtIndex(pathView.currentIndex, PathView.SnapPosition)

                handle.y = mouse.y

                if(moving)
                    handle.positionViewAtValue()
            }
        }

        Rectangle {
            id: groove
            anchors.centerIn: parent
            width: parent.width * 0.2; height: parent.height - handle.height
            color: styles.colors.white;
            radius: 2 * dpiRatio
            border { width: 1; color: styles.colors.whiteB }
        }

        FlyShadow { target: handle; horizontalOffset: handleMouseArea.pressed ? 0 : -2; verticalOffset: handleMouseArea.pressed ? 0 : 2 }
        Rectangle {
            id: handle

            property int value: pathView.count / (scrollBar.height - handle.width) * y

            function positionViewAtValue() {
                if(value >= pathView.count) {
                    pathView.positionViewAtIndex(pathView.count - 1, PathView.SnapPosition)
                } else {
                    pathView.positionViewAtIndex(value, PathView.SnapPosition)
                }
            }

            anchors { horizontalCenter: parent.horizontalCenter }
            width: parent.width; height: width
            color: styles.highlightColor; radius: width / 2

            MouseArea {
                id: handleMouseArea
                anchors.fill: parent
                onPressed: if(pathView.moving) pathView.positionViewAtIndex(pathView.currentIndex, PathView.SnapPosition)
                drag {
                    target: handle;
                    minimumY: 0;
                    maximumY: scrollBar.height - handle.width;
                    axis: Drag.YAxis;
                }
            }

            onValueChanged: {
                if(pathView.moving)
                    return

                positionViewAtValue()
            }
        }
    }
}
