//pragma Singleton
import QtQuick 2.3

SystemPalette {
    id: style
    colorGroup: SystemPalette.Active

    readonly property string settingsGroup: "Decoration\\Theme"

    readonly property var defaultColors: { "white": "#FFF",
                                           "gray": "#808080",
                                           "black": "#000",
                                           "red": "#F00",
                                           "green": "#0F0",
                                           "blue": "#00F" }

    // TODO delete this property
    readonly property bool isDefaults: Qt.colorEqual(highlightColor, themes[currentTheme].highlightColor) &&
                                       Qt.colorEqual(highlightColor2, themes[currentTheme].highlightColor2) &&
                                       Qt.colorEqual(colors.white, defaultColors.white) &&
                                       Qt.colorEqual(colors.gray, defaultColors.gray) &&
                                       Qt.colorEqual(colors.black, defaultColors.black) &&
                                       Qt.colorEqual(colors.red, defaultColors.red) &&
                                       Qt.colorEqual(colors.green, defaultColors.green) &&
                                       Qt.colorEqual(colors.blue, defaultColors.blue)

    property int currentTheme: settings.getInt("current-theme", settingsGroup, 0)
    readonly property var themes: [
        {
            "name": qsTr("Default"),
            "highlightColor": "#8EA1CC", // #94A7E2
            "highlightColor2": "#C3CEF0"
        },
        {
            "name": qsTr("Cyan"),
            "highlightColor": "#208888",
            "highlightColor2": "#99d8d0"
        },
        {
            "name": qsTr("Green"),
            "highlightColor": "#308830",
            "highlightColor2": "#99cf99"
        },
        {
            "name": qsTr("Magenta"),
            "highlightColor": "#883088",
            "highlightColor2": "#cf99cf"
        },
        {
            "name": qsTr("Gray"),
            "highlightColor": "#666666",
            "highlightColor2": "#cfcfcf"
        }/*,
        {
            "name": qsTr("Dark"),
            "highlightColor": "#666666",
            "highlightColor2": "#cfcfcf",
            "white": "#333333",
            "gray": "#555555",
            "black": "#dddddd"
        }*/
    ]

    property color highlightColor: settings.getString("dark", settingsGroup, themes[currentTheme].highlightColor)
    property color highlightColor2: settings.getString("light", settingsGroup, themes[currentTheme].highlightColor2)

    property QtObject colors: QtObject {
        property color  white: settings.getString("white", settingsGroup, defaultColors.white)
        property color  almostWhite: Qt.tint(white, "#0F000000") // "#f0f0f0"
        property color  whiteE: Qt.tint(white, "#11000000") // "#eee"
        property color  whiteD: Qt.tint(white, "#22000000") // "#ddd"
        property color  whiteC: Qt.tint(white, "#33000000") // "#ccc"
        property color  whiteB: Qt.tint(white, "#44000000") // "#bbb"
        property color  whiteA: Qt.tint(white, "#55000000") // "#aaa"
        property color  lightgray: Qt.tint(white, "#2C000000") // "#d3d3d3" or "lightgray"

        property color  gray: settings.getString("gray", settingsGroup, defaultColors.gray)
        property color  gray6: Qt.tint(gray, "#34000000") // "#666"
        property color  gray8: Qt.tint(gray, "#10FFFFFF") // "#888"
        property color  gray9: Qt.tint(gray, "#31FFFFFF") // "#999"

        property color  black: settings.getString("black", settingsGroup, defaultColors.black)
        property color  black1: Qt.tint(black, "#11FFFFFF") // "#111"
        property color  black2: Qt.tint(black, "#22FFFFFF") // "#222"
        property color  black3: Qt.tint(black, "#33FFFFFF") // "#333"
        property color  black4: Qt.tint(black, "#44FFFFFF") // "#444"
        property color  black5: Qt.tint(black, "#55FFFFFF") // "#555"

        property color  red:   settings.getString("red", settingsGroup, defaultColors.red)
        property color  green: settings.getString("green", settingsGroup, defaultColors.green)
        property color  blue:  settings.getString("blue", settingsGroup, defaultColors.blue)

        property color  yellow: root.getColorSumString(red, green) // "#FFFF00"
        property color  magenta: root.getColorSumString(red, blue) // "#FF00FF"
        property color  cyan: root.getColorSumString(green, blue) // "#00FFFF"

        property color  limegreen: Qt.tint(green, "#32FF00FF") // "#32cd32" or "limegreen"
        property color  crimson: Qt.tint(red, "#3C6A55FF") // "#dc143c" or "crimson"
        property color  darkmagenta: Qt.tint(magenta, "#74000000") // "#8b008b" or "darkmagenta"
        property color  limegreenLight: Qt.tint(green, "#A6F3FFFF") // "#9EFFA6"
        property color  darkred: Qt.tint(red, "#74000000") // "#8b0000" or "darkred"
        property color  redLight: Qt.tint(red, "#A0FFFFFC") // "#FFA09E"
        property color  indianred: Qt.tint(red, "#5C74FFFF") // "#cd5c5c" or "indianred"
        property color  blueLight: Qt.tint(blue, "#AA00FFFF") // "#00aaff"
        property color  honeydew: Qt.tint(green, "#F0FFFFFF") // "#f0fff0" or "honeydew"
        property color  peachpuff: Qt.tint(yellow, "#B9FFCBFF") // "#ffdab9" or "peachpuff"
        property color  palegreen: Qt.tint(green, "#98FFF8FF") // "#98fb98" or "palegreen"
        property color  ivory: Qt.tint(white, "#55FFFFD0") // "#fffff0" or "ivory"
    }

    property QtObject image: QtObject {
        property bool   mipmap: false
    }

    // TODO remove extra color bindings
    property QtObject page: QtObject {
        property color  backgroundColor: root.getColorString(0x55, colors.black)
        property color  foregroundColor: colors.almostWhite
        property color  centerRectColor: colors.white
        property color  titleColor: highlightColor
        property color  borderColor: colors.gray9
        property real   radius: 2 * dpiRatio
        property int    borderWidth: 1
    }

    property QtObject label: QtObject {
        property color  color: colors.black4
        property color  colorDisabled: colors.gray9
        property int    tiny: 14 * dpiRatio
        property int    smallest: 17 * dpiRatio
        property int    small: 20 * dpiRatio
        property int    medium: 22 * dpiRatio
        property int    big: 26 * dpiRatio
    }

    property QtObject buttons: QtObject {
        property color  colorLight: colors.almostWhite
        property color  colorDark: colors.whiteD
        property real   radius: 0
        property int    borderWidth: 0
    }

    property QtObject shadows: QtObject {
        property color  color: root.getColorString(0x66, colors.black)
        property color  darkColor: root.getColorString(0xAA, colors.black)
    }

    property QtObject separator: QtObject {
        property color  colorDark: root.getColorString(0x25, colors.black)
        property color  colorLight: root.getColorString(0x55, colors.white)
    }

    property QtObject lineedit: QtObject {
        property color  borderColor: placeholderColor
        property color  textColor: label.color
        property color  placeholderColor: colors.gray8
    }

    property QtObject listView: QtObject {
        property color  delegateColor: colors.white
        property color  backgroundColor: colors.white
    }

    property QtObject calendar: QtObject {
        property color  color: colors.white
        property color  headerColor: colors.almostWhite
        property color  greyDayColor: colors.gray
        property color  weekDayColor: colors.black2
        property color  weekendDayColor: colors.red
        property color  handleColor: colors.crimson
        property color  lineHandleBorderColor: colors.whiteE
        property var    lineHandleGradient: Gradient {
            GradientStop { position: 0; color: root.getColorString(0x22, colors.whiteE) }
            GradientStop { position: 1.0; color: root.getColorString(0x88, colors.whiteE) }
        }
        property var    yearListGradient: Gradient {
            GradientStop { position: 0.8; color: colors.white }
            GradientStop { position: 1.0; color: colors.whiteA }
        }
    }

    property QtObject radioButtonRow: QtObject {
        property color  borderColor: colors.gray
        property color  buttonColor: colors.white
    }

    property QtObject touchAnimation: QtObject {
        property color  back:   root.getColorString(0x66, highlightColor2)
        property color  front:  highlightColor2
    }

    property QtObject systemBars: QtObject {
        property color  statusBarColor: "#000"
        property color  buttonBarColor: "#000"
        property color  textColor:      "#eee"
    }

    function setTheme(theme) {
        currentTheme = theme;
        restoreDefaults()
    }

    function restoreDefaults() {
        settings.removeKey(settingsGroup.split("\\")[1], settingsGroup.split("\\")[0])
        settings.setInt("current-theme", currentTheme, settingsGroup)

        highlightColor = Qt.binding(function() {return settings.getString("dark", settingsGroup, themes[currentTheme].highlightColor) })
        highlightColor2 = Qt.binding(function() {return settings.getString("light", settingsGroup, themes[currentTheme].highlightColor2) })
        colors.white = Qt.binding(function() {return settings.getString("white", settingsGroup, defaultColors.white) })
        colors.gray = Qt.binding(function() {return settings.getString("gray", settingsGroup, defaultColors.gray) })
        colors.black = Qt.binding(function() {return settings.getString("black", settingsGroup, defaultColors.black) })
        colors.red = Qt.binding(function() {return settings.getString("red", settingsGroup, defaultColors.red) })
        colors.green = Qt.binding(function() {return settings.getString("green", settingsGroup, defaultColors.green) })
        colors.blue = Qt.binding(function() {return settings.getString("blue", settingsGroup, defaultColors.blue) })
    }
}
