import QtQuick 2.3

Item {
    id: switchBox

    property bool on: true
    property color onColor: styles.highlightColor
    property color offColor: styles.colors.whiteD
    property color handleColor: styles.colors.white
    signal valueChanged(bool on)

    width: height * 1.5; height: root.itemHeight * 0.8

    function setOn(_on) {
        if(on == _on) return;
        on = _on;
        valueChanged(on);
    }

    MouseArea {
        anchors.fill: parent
        onClicked: setOn(!on)
    }

    Rectangle {
        id: groove
        anchors.fill: parent
        color: on ? onColor : offColor
        radius: height / 2
        border { width: 1; color: Qt.darker(color, 1.1) }

        Behavior on color { ColorAnimation {} }
    }

    FlyShadow { target: handle; horizontalOffset: 0; verticalOffset: 1; }
    Rectangle {
        id: handle
        width: height; height: parent.height * 0.8
        anchors.verticalCenter: parent.verticalCenter
        color: handleColor
        radius: height / 2

        Binding on x {
            when: !mouseArea.drag.active;
            value: on ? mouseArea.drag.maximumX : mouseArea.drag.minimumX
        }

        MouseArea {
            id: mouseArea
            anchors.fill: parent
            drag {
                target: handle
                axis: Drag.XAxis
                minimumX: (switchBox.height - height) / 2
                maximumX: switchBox.width - width - (switchBox.height - height) / 2
            }

            onClicked: setOn(!on)
        }

        onXChanged: if(mouseArea.drag.active) {
                        if(!on && handle.x > mouseArea.drag.minimumX + (mouseArea.drag.maximumX - mouseArea.drag.minimumX) / 2) {
                            setOn(true);
                        } else if(on && handle.x <= mouseArea.drag.minimumX + (mouseArea.drag.maximumX - mouseArea.drag.minimumX) / 2) {
                            setOn(false);
                        }
                    }

        Behavior on x { NumberAnimation {} }
    }
}
