import QtQuick 2.3

TextEdit {
    id: text

    property alias placeholderText: textEditPlaceholder.text
    property alias flickable: selection.flickable

    color: styles.lineedit.textColor
    font.pixelSize: styles.label.small
    wrapMode: Text.Wrap
    textFormat: TextEdit.PlainText
    selectByMouse: isDesktopMode
    selectedTextColor: styles.label.color
    selectionColor: styles.highlightColor2
    focus: true

    Accessible.role: Accessible.EditableText
    Accessible.multiLine: true
    Accessible.name: "Text"

    FlyLabel {
        id: textEditPlaceholder
        anchors.fill: parent; anchors.leftMargin: 5
        text: ""
        color: styles.lineedit.placeholderColor
        visible: (text.text === "" && !text.activeFocus)
        font {
            italic: true
            pixelSize: text.font.pixelSize;
        }
        verticalAlignment: TextInput.AlignTop
    }

    MouseArea {
        anchors.fill: parent
        cursorShape: parent.hoveredLink ? Qt.PointingHandCursor : Qt.ArrowCursor
        acceptedButtons: Qt.NoButton
    }

    FlyMouseAreaSelection { id: selection; inputTarget: text }

    onLinkActivated: {
        if(debug) console.log("Opening url:", link);

        Qt.openUrlExternally(link)
    }

    onCursorPositionChanged: {
        if(!flickable || y + contentHeight < flickable.height)
            return

        if(cursorRectangle.y < flickable.contentY) {
            flickable.contentY = cursorRectangle.y
        } else if(cursorRectangle.y + cursorRectangle.height > flickable.contentY + flickable.height) {
            flickable.contentY = cursorRectangle.y + cursorRectangle.height - flickable.height
        }
    }

    onActiveFocusChanged: if(activeFocus && root.virtualKeyboard) root.virtualKeyboard.setActiveItem(text, true)
}
