import QtQuick 2.3

Column {
    id: timePicker

    readonly property string time: normalize(hour) + ":" + normalize(min)
    property int hour: 0
    property int min: 0
    property int sec: 0
    property bool isHourChoosing: true
    //property alias isPm: amPmSwitch.on

    function normalize(time) {
        return time < 10 ? "0" + time : time;
    }

//    Row {
        Rectangle {
            id: timeLabelRect
            width: timePicker.width; height: Math.min(timePicker.height * 0.3, root.itemHeight * 2)
            color: styles.highlightColor

            Row {
                anchors.centerIn: parent

                FlyLabel {
                    text: normalize(hour)
                    font.pixelSize: timeLabelRect.height * 0.5
                    color: styles.colors.white
                    opacity: isHourChoosing ? 1 : 0.7

                    MouseArea { anchors.fill: parent; onClicked: { isHourChoosing = true; canvas.repaint(); } }
                }

                FlyLabel {
                    text: ":"
                    font.pixelSize: timeLabelRect.height * 0.5
                    color: styles.colors.white
                }

                FlyLabel {
                    text: normalize(min)
                    font.pixelSize: timeLabelRect.height * 0.5
                    color: styles.colors.white
                    opacity: !isHourChoosing ? 1 : 0.7

                    MouseArea { anchors.fill: parent; onClicked: { isHourChoosing = false; canvas.repaint(); } }
                }
            }
        }

//        Rectangle {
//            id: amPmRect
//            width: timePicker.width / 2; height: timeLabelRect.height
//            color: "transparent"
//            border { width: 1; color: timeLabelRect.color }

//            Row {
//                anchors.centerIn: parent

//                FlyLabel {
//                    text: "AM "
//                    font.pixelSize: amPmRect.height * 0.4
//                    color: amPmSwitch.on ? styles.label.colorDisabled : styles.label.color

//                    MouseArea { anchors.fill: parent; onClicked: { amPmSwitch.setOn(false); canvas.repaint(); } }
//                }

//                FlySwitchBox {
//                    id: amPmSwitch;
//                    anchors.verticalCenter: parent.verticalCenter;
//                    on: hour >= 12
//                    onValueChanged: { hour = (hour + (on ? 12 : -12)) % 24; canvas.repaint(); }
//                }

//                FlyLabel {
//                    text: " PM"
//                    font.pixelSize: amPmRect.height * 0.4
//                    color: !amPmSwitch.on ? styles.label.colorDisabled : styles.label.color

//                    MouseArea { anchors.fill: parent; onClicked: { amPmSwitch.setOn(true); canvas.repaint(); } }
//                }
//            }
//        }
//    }

    Item {
        id: clock
        width: parent.width; height: parent.height - timeLabelRect.height

        Canvas {
            id: canvas

            readonly property color darkBorderColor: root.getColorSumString(styles.colors.black, "#767676")

            width: Math.min(parent.width, parent.height) * 0.95; height: width
            anchors.centerIn: parent

            function repaint() {
                requestPaint();
                canvasArrows.requestPaint();
            }

            onPaint: {
                var ctx = getContext("2d")
                var radius = Math.min(width, height) / 2 * 0.9;
                var hw = width / 2
                var hh = height / 2

                ctx.clearRect(0, 0, width, height)
                drawClock();

                function drawClock() {
                    drawFace();
                    drawMarks();
                    drawNumbers();
                }

                function drawFace() {
                    ctx.save()
                    ctx.beginPath();
                    ctx.arc(hw, hh, radius, 0 , Math.PI * 2);
                    var grad = ctx.createRadialGradient(hw, 0, radius * 0.5, hw, hh, radius * 0.6);
                    grad.addColorStop(0, styles.colors.white);
                    grad.addColorStop(1, Qt.darker(styles.colors.white, 1.3));

                    ctx.fillStyle = grad;
                    ctx.fill();

                    grad = ctx.createRadialGradient(hw, hh, 0, hw, hh, radius * 1.05);
                    grad.addColorStop(0.91, darkBorderColor);
                    grad.addColorStop(0.93, styles.colors.white);
                    grad.addColorStop(0.97, styles.colors.white);
                    grad.addColorStop(1, darkBorderColor);

                    ctx.strokeStyle = grad;
                    ctx.lineWidth = radius * 0.1;
                    ctx.stroke();
                    ctx.restore();
                }

                function drawMarks() {
                    ctx.strokeStyle = styles.colors.black;

                    for(var i = 0; i < 60; i++) {
                        var theta = i * Math.PI / 30;
                        ctx.lineWidth = i % 5 ? 1 : 3;
                        ctx.beginPath();
                        ctx.moveTo(hw + radius * 0.95 * Math.cos(theta), hh + radius * 0.95 * Math.sin(theta));
                        ctx.lineTo(hw + radius * 0.9 * Math.cos(theta), hh + radius * 0.9 * Math.sin(theta));
                        ctx.stroke();
                    }
                }

                function drawNumbers() {
                    ctx.save();
                    ctx.translate(canvas.width / 2, canvas.height / 2);
                    ctx.fillStyle = isHourChoosing ? styles.colors.gray : styles.colors.black1;
                    ctx.textAlign = 'center';
                    ctx.textBaseline = 'middle';

                    for (var n = 0; n <= 59; n++) {
                        ctx.font = (!(n % 5) ? "bold " + radius * 0.06 : radius * 0.05) + "px " + defaultFont;
                        if(n > 0) ctx.rotate(6 * (Math.PI / 180));
                        var x = 0;
                        var y = -hh * 0.89;
                        ctx.fillText(n, x, y);
                    }

                    ctx.restore();
                }
            }
        }

        Item {
            id: clockNumbers
            width: Math.min(parent.width, parent.height) * 0.68; height: width
            anchors.centerIn: parent

//            Rectangle {
//                id: hourBgRect
//                width: root.itemHeight / 2; height: width
//                radius: width / 2
//                color: root.getColorString(0x66, styles.colors.white)
//            }

            Repeater {
                anchors.fill: parent
                model: 12

                Item {
                    width: root.itemHeight; height: width
                    x: clockNumbers.width / 2 + Math.cos(360 / 12 * (index - 2) * Math.PI / 180) * (clockNumbers.width / 2) - width / 2
                    y: clockNumbers.height / 2 + Math.sin(360 / 12 * (index - 2) * Math.PI / 180) * (clockNumbers.height / 2) - height / 2

                    FlyLabel {
                        id: clockLabel
                        anchors.centerIn: parent
                        text: index + 1
                        color: isHourChoosing ? styles.colors.black3 : styles.colors.gray9;
                    }

                    MouseArea {
                        anchors.fill: parent; anchors.margins: parent.width * 0.1
                        enabled: isHourChoosing
                        onClicked: {
                            hour = clockLabel.text;
                            isHourChoosing = false;

                            //hourBgRect.x = parent.x + parent.width * 0.25; hourBgRect.y = parent.y + parent.width * 0.25;

                            canvas.repaint();
                        }
                    }
                }
            }

            Repeater {
                anchors.fill: parent
                model: 12

                Item {
                    width: root.itemHeight * 0.7; height: width
                    x: clockNumbers.width / 2 + Math.cos(360 / 12 * (index - 2) * Math.PI / 180) * (clockNumbers.width / 2 - 30 * dpiRatio) - width / 2
                    y: clockNumbers.height / 2 + Math.sin(360 / 12 * (index - 2) * Math.PI / 180) * (clockNumbers.height / 2 - 30 * dpiRatio) - height / 2

                    FlyLabel {
                        id: clockLabel2
                        anchors.centerIn: parent
                        text: index + 13 === 24 ? "00" : (index + 13) % 24
                        color: isHourChoosing ? styles.colors.black3 : styles.colors.gray9;
                    }

                    MouseArea {
                        anchors.fill: parent; anchors.margins: parent.width * 0.1
                        enabled: isHourChoosing
                        onClicked: {
                            hour = clockLabel2.text;
                            isHourChoosing = false;

                            canvas.repaint();
                        }
                    }
                }
            }

            MouseArea {
                anchors.fill: parent
                enabled: !isHourChoosing
                drag.target: clockNumbers // just need this so clicked signal won't be emitted after dragging

                onClicked: {
                    var mins = coordsToMins(mouse.x, mouse.y);
                    if(mins % 5 === 4) { mins = (mins + 1) % 60; } // just rounding adjacent minutes to *0 or *5
                    else if(mins % 5 === 1) { mins--; }

                    if(mins !== min) {
                        min = mins;
                        canvas.repaint();
                    }
                }
                onPositionChanged: {
                    var mins = coordsToMins(mouse.x, mouse.y);
                    if(mins !== min) {
                        min = mins;
                        canvas.repaint();
                    }
                }

                function coordsToMins(x, y) {
                    var rad = Math.atan2(clockNumbers.height / 2 - y, clockNumbers.width / 2 - x);
                    var deg = rad * (180 / Math.PI) + 180;
                    return Math.round(deg / 6 + 15) % 60;
                }
            }
        }

        Canvas {
            id: canvasArrows
            anchors.fill: canvas

            onPaint: {
                var ctx = getContext("2d")
                var radius = Math.min(width, height) / 2 * 0.9;

                ctx.clearRect(0, 0, width, height)

                drawHand(min * Math.PI / 30, radius * 0.75, radius * 0.0375, styles.colors.black);
                drawHand(hour % 12 * Math.PI / 6, radius * 0.5, radius * 0.05, styles.colors.black);

                function drawHand(pos, length, width, style) {
                    ctx.save()
                    ctx.fillStyle = style;
                    ctx.translate(canvas.width / 2, canvas.height / 2);
                    ctx.rotate(pos);
                    ctx.beginPath();
                    ctx.moveTo(Math.floor(-width / 2), length * 0.25);
                    ctx.lineTo(Math.floor(width / 2), length * 0.25);
                    ctx.lineTo(Math.floor(width / 5), -length);
                    ctx.lineTo(Math.floor(-width / 5), -length);
                    ctx.fill();
                    ctx.restore()
                }
            }
        }
    }
}
