import QtQuick 2.3

Item {
    id: timePicker

    readonly property string time: normalize(hour) + ":" + normalize(min) + ":" + normalize(sec)
    property var timeModel: [24, 60, 60]
    property int hour: 0
    property int min: 0
    property int sec: 0

    function normalize(time) {
        return time < 10 ? "0" + time : time;
    }

    Rectangle {
        anchors.horizontalCenter: parent.horizontalCenter
        y: parent.height * 0.42
        width: parent.width * 0.9
        height: 1
        color: styles.highlightColor
    }

    Rectangle {
        anchors.horizontalCenter: parent.horizontalCenter
        y: parent.height * 0.58
        width: parent.width * 0.9
        height: 1
        color: styles.highlightColor
    }

    FlyLabel {
        anchors.verticalCenter: parent.verticalCenter
        x: parent.width * 0.33
        text: ":"
        font.pixelSize: styles.label.big
    }

    FlyLabel {
        anchors.verticalCenter: parent.verticalCenter
        x: parent.width * 0.66
        text: ":"
        font.pixelSize: styles.label.big
    }

    Row {
        anchors.fill: parent

        Repeater {
            id: repeater
            model: timeModel.length

            PathView {
                id: pathView
                readonly property real itemHeight: height / pathItemCount
                width: timePicker.width / repeater.count; height: parent.height
                model: timeModel[index]
                currentIndex: index === 0 ? hour : index === 1 ? min : sec
                pathItemCount: 5
                preferredHighlightBegin: 0.5; preferredHighlightEnd: 0.5;
                delegate: FlyLabel {
                    width: pathView.width; height: pathView.itemHeight
                    text: normalize(modelData)
                    font.pixelSize: styles.label.big
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                    opacity: PathView.iconOpacity
                }
                path: Path {
                    startX: pathView.width / 2; startY: 0 + pathView.itemHeight / 2
                    PathAttribute { name: "iconOpacity"; value: 0 }
                    PathLine { x: pathView.width / 2; y: pathView.height / 2 - pathView.itemHeight / 2 }
                    PathAttribute { name: "iconOpacity"; value: 1 }
                    PathLine { x: pathView.width / 2; y: pathView.height - pathView.itemHeight / 2 }
                    PathAttribute { name: "iconOpacity"; value: 0 }
                }

                onCurrentIndexChanged: switch(index) {
                                       case 0: if(hour !== currentIndex) hour = currentIndex; break;
                                       case 1: if(min  !== currentIndex) min = currentIndex;  break;
                                       case 2: if(sec  !== currentIndex) sec = currentIndex;  break;
                                       }
            }

//            Tumbler { // TODO import QtQuick.Controls 2.0
//                width: timePicker.width / repeater.count; height: parent.height
//                model: timeModel[index]
//                currentIndex: index === 0 ? hour : index === 1 ? min : sec
//                visibleItemCount: 5

//                delegate: FlyLabel {
//                    text: normalize(modelData)
//                    font.pixelSize: styles.label.big
//                    horizontalAlignment: Text.AlignHCenter
//                    verticalAlignment: Text.AlignVCenter
//                    opacity: 1.0 - Math.abs(Tumbler.displacement) / (visibleItemCount / 2)
//                }

//                onCurrentIndexChanged: switch(index) {
//                                       case 0: if(hour !== currentIndex) hour = currentIndex; break;
//                                       case 1: if(min  !== currentIndex) min = currentIndex;  break;
//                                       case 2: if(sec  !== currentIndex) sec = currentIndex;  break;
//                                       }
//            }
//        }
        }
    }
}
