import QtQuick 2.3

Item {
    id: toolBar

    /* TODO:
      ✓ 1.  left row items
        1.1 autohide left items too
      x 2.  FlyPage integration (via Loader?)
      ✓ 3.  test client integration
      ✓ 4.  right row items
      x 5.  default items ???
      ✓ 6.  autohide items (on resize, startup, model change)
      ✓ 7.  create filter line edit
        7.1 add custom elements
      ✓ 8.  context menu
      ✓ 8.1 context menu static and dynamic models (staticModel.concat(getDynamicModelNames()))
      ✓ 8.2 always show context menu if there is staticModel
      ✓ 9.  autoshow in context menu
      ✓ 10. create headerOkCancel
      ✓ 10.1 use back button to return from helper toolbar
        10.2 add custom elements
      ✓ 11. all apps integration
      ✓ 11.1 fly-contacts
      ✓ 11.2 fly-mail
      ✓ 11.3 fly-notes
      ✓ 11.4 fly-qml-dialer
      ✓ 11.5 fly-sms
      ✓ 12. search state -> helper state -> search state
    */

    // NOTE: all items in right row must have menuName and clicked signal or function!!!

    // main toolbar
    property alias leftRowItems: leftRow.children
    property alias rightRowItems: rightRow.children

    property var menuStaticModel: []
    signal menuStaticClicked(int index)

    // helper toolbar
    readonly property alias backButton: backButton
    readonly property alias okButton: okButton

    // search toolbar
    readonly property alias searchButton: searchButton
    readonly property alias searchLine: searchLineEdit

    height: Math.min(parent.height * 0.1, 60 * dpiRatio)
    z: 1
    enabled: visible

    function toggleHelperBar() {
        if(state === "helper") {
            root.popFunctionFromBackStack();
            state = "";
        } else if(state === "searchhelper") {
            root.popFunctionFromBackStack();
            state = "search";
        } else if(state === "search") {
            root.addFunctionToBackStack(toggleHelperBar);
            state = "searchhelper";
        } else {
            root.addFunctionToBackStack(toggleHelperBar);
            state = "helper";
        }
    }

    function toggleSearchBar() {
        if(state === "search") {
            root.popFunctionFromBackStack();
            state = "";

            searchLineEdit.text = "";
            searchLineEdit.focus = false;
        } else {
            root.addFunctionToBackStack(toggleSearchBar);
            state = "search";

            searchLineEdit.focus = true;
        }
    }

    MouseArea { anchors.fill: parent }
    FlyShadow { target: toolBarRect; horizontalOffset: 0; verticalOffset: 1; }
    Rectangle {
        id: toolBarRect

        readonly property real freeSpace: width - rightRow.width - leftRow.width - (searchButton.visible ? searchButton.width : 0)

        anchors.fill: parent
        color: styles.page.titleColor

        function resetLayout() {
            if(resetLayoutTimer.running) return;

            if(toolBarRect.freeSpace < 0 && rightRow.children.length > 1) {
                var count = menuButton.visible ? 1 : Math.min(2, rightRow.children.length);
                var spaceLeft = Math.abs(toolBarRect.freeSpace) // calculate overlimit width
                        - rightRow.children[rightRow.children.length - 1].width // left item width
                        + (!menuButton.visible
                           ? menuButton.width
                             - (rightRow.children.length >= 2 // maybe it's useless check because there's need to be 2 items to menu to appear
                                ? rightRow.children[rightRow.children.length - 2].width
                                : 0)
                           : 0); // just take only one item width

                // if screen orientation changed, so we need several item movement at once...
                while(rightRow.children.length - 1 > count && spaceLeft >= 0) { // -1 is for menu button
                    spaceLeft -= rightRow.children[rightRow.children.length - 1 - count].width;
                    count++;
                }

                for(var i = 0; i < count; i++)
                    rightRow.children[rightRow.children.length - 1].parent = dummy;

                if(!menuButton.visible) menuButton.visible = true;

                resetLayoutTimer.start();
            } else if(dummy.children.length > 0
                      && toolBarRect.freeSpace >= dummy.children[dummy.children.length - 1].width) {
                count = dummy.children.length > 2 || menuStaticModel.length > 0 ? 1 : dummy.children.length;
                spaceLeft = toolBarRect.freeSpace - dummy.children[dummy.children.length - 1].width;

                // if screen orientation changed, so we need several item movement at once...
                while(dummy.children.length > count
                      && spaceLeft >= dummy.children[dummy.children.length - 1 - count].width) {
                    spaceLeft -= dummy.children[dummy.children.length - 1 - count].width;
                    count++;
                }

                if(count >= dummy.children.length && menuStaticModel.length === 0) menuButton.visible = false;

                for(i = 0; i < count; i++)
                    dummy.children[dummy.children.length - 1].parent = rightRow;

                resetLayoutTimer.start();
            }
        }

        Timer { id: resetLayoutTimer; onTriggered: toolBarRect.resetLayout(); }

        // this item is needed for autohiding menuitems from rightrow in toolBar if they don't have enough space
        // they reparented to dummy
        // for menu they should contain 'menuName' property and 'clicked' signal or function
        Item { id: dummy; width: 200; height: parent.height; visible: false }

        Row { id: leftRow; anchors { left: parent.left; top: parent.top; bottom: parent.bottom } }

        FlyMenuButton {
            id: searchButton
            anchors.left: leftRow.right
            image.imagePath: searchLineEdit.visible ? "qrc:/images/flaticon/previous.svg" : "qrc:/images/flaticon/search-button.svg"
            image.imageSourceSize: Qt.size(width, height)
            onClicked: toggleSearchBar();
        }

        FlyLineEdit {
            id: searchLineEdit
            anchors { left: searchButton.right; top: parent.top; right: rightRow.left; bottom: parent.bottom
                leftMargin: 10 * dpiRatio; topMargin: parent.height * 0.1; rightMargin: anchors.leftMargin; bottomMargin: anchors.topMargin }
            showClearButton: true
            visible: false; enabled: visible
            Keys.onEnterPressed: toggleSearchBar();
            Keys.onEscapePressed: toggleSearchBar();
            onEnterClicked: toggleSearchBar();
            Component.onCompleted: {
                if(Qt.colorEqual(toolBarRect.color, styles.highlightColor)) {
                    searchLineEdit.textColor = styles.colors.white;
                    searchLineEdit.placeholderTextColor = styles.colors.whiteC;

                    if("linesColor" in searchLineEdit) searchLineEdit.linesColor = Qt.binding(function() {
                        return searchLineEdit.activeFocus ? styles.highlightColor2 : styles.colors.whiteC })
                }
            }
        }

        Row {
            id: rightRow
            anchors { top: parent.top; right: parent.right; bottom: parent.bottom }
            layoutDirection: Qt.RightToLeft

            FlyMenuButton {
                id: menuButton
                image.imagePath: "qrc:/images/menu.svg"
                image.imageSourceSize: Qt.size(width, height)
                visible: menuStaticModel.length > 0
                onClicked: { // TODO HEAVY every time it creates model and calcs things
                    var model = menuStaticModel.concat(getDynamicModelNames());

                    contextMenu.menuRect.anchors.right = contextMenu.menuRect.parent.right;
                    contextMenu.menuRect.anchors.rightMargin = ("frontRect" in toolBar.parent && toolBar.parent.frontRect.visible
                                                                ? toolBar.parent.frontRect.x : 0);
                    contextMenu.menuRect.y = toolBar.height + toolBar.y + ("frontRect" in toolBar.parent && toolBar.parent.frontRect.visible
                                                                           ? toolBar.parent.frontRect.y : 0);
                    contextMenu.show(model, onContextMenuClicked);
                }

                function getDynamicModelNames() {
                    var model = [];
                    for(var i = 0; i < dummy.children.length; i++)
                        if(dummy.children[i].menuName) model.push(dummy.children[i].menuName);
                    return model;
                }

                function onContextMenuClicked(index) {
                    if(index >= 0) {
                        if(index < menuStaticModel.length) {
                            menuStaticClicked(index);
                        } else {
                            dummy.children[index - menuStaticModel.length].clicked();
                        }
                    }
                }
            }
        }

        onFreeSpaceChanged: toolBarRect.resetLayout();
    }

    Rectangle {
        id: helperBarRect
        width: parent.width; height: parent.height
        y: -height
        color: toolBarRect.color
        enabled: false

        Row {
            anchors { left: parent.left; top: parent.top; bottom: parent.bottom }

            FlyMenuButton {
                id: backButton
                image.imagePath: "qrc:/images/flaticon/previous.svg"
                image.imageSourceSize: Qt.size(width, height)
                onClicked: toggleHelperBar()
            }
        }

        Row {
            anchors { top: parent.top; right: parent.right; bottom: parent.bottom }

            FlyMenuButton {
                id: okButton
                image.imagePath: "qrc:/images/flaticon/check-mark.svg"
                image.imageSourceSize: Qt.size(width, height)
                onClicked: toggleHelperBar()
            }
        }
    }

    states: [
        State {
            name: "helper"
            PropertyChanges { target: helperBarRect; y: 0; enabled: true }
            PropertyChanges { target: toolBarRect; enabled: false }
        },
        State {
            name: "search"
            PropertyChanges { target: leftRow; anchors.leftMargin: -leftRow.width }
            PropertyChanges { target: rightRow; anchors.rightMargin: -rightRow.width }
            PropertyChanges { target: searchLineEdit; visible: true }
        },
        State {
            name: "searchhelper"
            PropertyChanges { target: helperBarRect; y: 0; enabled: true }
            PropertyChanges { target: toolBarRect; enabled: false }
            PropertyChanges { target: leftRow; anchors.leftMargin: -leftRow.width }
            PropertyChanges { target: rightRow; anchors.rightMargin: -rightRow.width }
            PropertyChanges { target: searchLineEdit; visible: true }
        }
    ]

    transitions: Transition {
        NumberAnimation { properties: "y,anchors.leftMargin,anchors.rightMargin"; duration: 350; easing.type: Easing.InOutQuad }
    }
}
