import QtQuick 2.3
//import QtQuick.Enterprise.VirtualKeyboard 2.0 //1.3
//import QtQuick.Enterprise.VirtualKeyboard.Settings 2.0
import QtQuick.VirtualKeyboard 2.1 //1.3
import QtQuick.VirtualKeyboard.Settings 2.0
import Fly.Components 1.0

InputPanel {
    id: inputPanel

    width: root.width
    y: root.height
    z: 99
    visible: false
    enabled: visible

    function setEnglishLayout() {
        if(VirtualKeyboardSettings.locale !== "en_GB") VirtualKeyboardSettings.locale = "en_GB"
        keyboard.localeIndex = keyboard.defaultLocaleIndex
    }

    function setActiveItem(item, multiLine) {
        var delay = Date.now() - inputLoader.delayOpen;
        if(delay < 1000) return;
        if(item !== inputLoader.origInputItem) {
            inputLoader.origInputItem = item
            inputLoader.multiLine = multiLine
        }
    }

    function delayedInit() {
        if(inputLoader.active && state === "visible" && inputLoader.origInputItem) {
            var obj = inputLoader.origInputItem;
            var centerRect = null;
            var inputBottomLine = 0;

            if(root.currentPage) {
                centerRect = root.currentPage.centerRect;
                inputBottomLine = obj.mapToItem(root.currentPage, 0, obj.y).y + obj.height;
            }

//            console.warn("TESTT1", obj.height, "<=", root.itemHeight * 0.6)
//            console.warn("TESTT2", y, "<", obj.mapToItem(root.currentPage, 0, obj.y).y + obj.height)
//            console.warn("TESTT3", root.currentPage.centerRect.y + root.currentPage.centerRect.height, "<", obj.mapToItem(root.currentPage, 0, obj.y).y + obj.height)

            if((obj.height <= root.itemHeight * 0.5) // too small
                    || (root.currentPage
                        && ((y < inputBottomLine) // vkbd hovers input
                            || (centerRect.visible && (centerRect.y + centerRect.height < inputBottomLine))))) // container hovers input
                inputLoader.initLineEdit()
        }
    }

    //FlyVkbdDbusNotifier { onHide: { Qt.inputMethod.commit(); inputLoader.endLineEdit() } }

    Loader {
        id: inputLoader

        readonly property var inputText: item ? item.inputText : null
        property real delayOpen: 0
        property var origInputItem: null
        property bool multiLine: false
        property bool inited: false

        anchors.bottom: parent.top
        width: parent.width; height: root.height - inputPanel.height
        active: height < root.height * 0.4

        function initLineEdit() {
            if(!inputLoader.origInputItem || !inputLoader.active || !inputText) return

            Qt.inputMethod.commit()

            if(multiLine) inputText.wrapMode = inputLoader.origInputItem.wrapMode
            if(multiLine) inputText.textFormat = inputLoader.origInputItem.textFormat
            if(!multiLine) inputText.validator = inputLoader.origInputItem.validator
            if(!multiLine) inputText.echoMode = inputLoader.origInputItem.echoMode
            if(!multiLine) inputText.inputMask = inputLoader.origInputItem.inputMask
            inputText.inputMethodHints = inputLoader.origInputItem.inputMethodHints
            inputText.text = inputLoader.origInputItem.text
            inputText.cursorPosition = inputLoader.origInputItem.cursorPosition
            if(inputLoader.origInputItem.selectionStart !== inputLoader.origInputItem.selectionEnd)
                inputText.select(inputLoader.origInputItem.selectionStart, inputLoader.origInputItem.selectionEnd)
            inputLoader.inited = true

            delayFocus.start()
        }

        function endLineEdit() {
            if(!inputLoader.origInputItem || !inputLoader.active || !inputText || !inputLoader.inited) return

            inputLoader.origInputItem.text = inputText.text

            delayOpen = Date.now();
            inputText.focus = false
            inputLoader.origInputItem.forceActiveFocus()
            inputLoader.origInputItem = null
            inputLoader.inited = false

            if(Qt.inputMethod.visible) Qt.inputMethod.hide()
        }

        Timer { id: delayFocus; interval: 500; onTriggered: inputLoader.inputText.forceActiveFocus() }

        sourceComponent: Component {
            Rectangle {
                id: inputRect

                readonly property var inputText: inputLoader.multiLine ? multiLineText : lineText

                color: styles.page.foregroundColor
                visible: inputLoader.origInputItem && inputLoader.inited
                enabled: visible

                MouseArea { anchors.fill: parent }
                Row {
                    id: inputRow
                    anchors { fill: parent; margins: 20 * dpiRatio }
                    spacing: 20 * dpiRatio

                    Rectangle {
                        anchors.verticalCenter: parent.verticalCenter
                        width: parent.width - doneButton.width - inputRow.spacing
                        height: inputLoader.multiLine ? parent.height : root.itemHeight
                        color: styles.page.centerRectColor
                        border { width: 1; color: inputText.activeFocus ? styles.highlightColor : styles.colors.gray }
                        radius: 3
                        clip: true

                        TextInput {
                            id: lineText
                            anchors { fill: parent; leftMargin: 20 * dpiRatio; rightMargin: 20 * dpiRatio }
                            font.pixelSize: styles.label.small
                            verticalAlignment: TextInput.AlignVCenter
                            color: styles.lineedit.textColor
                            visible: !inputLoader.multiLine
                            enabled: visible
                            selectByMouse: isDesktopMode
                            selectedTextColor: styles.label.color
                            selectionColor: styles.highlightColor2
                            Accessible.role: Accessible.EditableText
                            Accessible.name: "Text"
                            Accessible.editable: true
                            EnterKeyAction.enabled: acceptableInput
                            EnterKeyAction.actionId: EnterKeyAction.Go
                            onAccepted: inputLoader.endLineEdit()
                        }
                        FlyMouseAreaSelection { inputTarget: lineText; enabled: !isDesktopMode && lineText.enabled }

                        Flickable { // TODO Loader
                            id: multiLineTextFlickable
                            anchors { fill: parent; margins: 20 * dpiRatio }
                            flickableDirection: Flickable.VerticalFlick
                            contentHeight: multiLineText.contentHeight
                            visible: inputLoader.multiLine
                            enabled: visible

                            TextEdit {
                                id: multiLineText
                                width: multiLineTextFlickable.width
                                height: Math.max(contentHeight, multiLineTextFlickable.height)
                                color: styles.lineedit.textColor
                                font.pixelSize: styles.label.small
                                selectByMouse: isDesktopMode
                                selectedTextColor: styles.label.color
                                selectionColor: styles.highlightColor2

                                Accessible.role: Accessible.EditableText
                                Accessible.multiLine: true
                                Accessible.name: "MultiLineText"

                                onCursorPositionChanged: {
                                    if(contentHeight < multiLineTextFlickable.height)
                                        return

                                    if(cursorRectangle.y < multiLineTextFlickable.contentY) {
                                        multiLineTextFlickable.contentY = cursorRectangle.y
                                    } else if(cursorRectangle.y + cursorRectangle.height > multiLineTextFlickable.contentY + multiLineTextFlickable.height) {
                                        multiLineTextFlickable.contentY = cursorRectangle.y + cursorRectangle.height - multiLineTextFlickable.height
                                    }
                                }

                                FlyMouseAreaSelection { inputTarget: multiLineText; flickable: multiLineTextFlickable }
                            }
                        }
                    }

                    FlyControlButton {
                        id: doneButton
                        height: root.itemHeight
                        anchors.verticalCenter: parent.verticalCenter
                        image.imagePath: "image://ThemeImageProvider/dialog-ok-apply"
                        onClicked: inputLoader.endLineEdit()
                    }
                }
            }
        }
    }

    states: State {
        name: "visible"
        when: Qt.inputMethod.visible
        PropertyChanges {
            target: inputPanel
            y: root.height - inputPanel.height
        }
    }

    transitions: Transition {
        from: ""
        to: "visible"
        reversible: true
        SequentialAnimation {
            ScriptAction { script: { inputPanel.visible = inputPanel.state === "visible"; inputLoader.origInputItem = null } }
            NumberAnimation {
                properties: "y"
                duration: 250
                easing.type: Easing.InOutQuad
            }
            ScriptAction { script: { delayedInit() } }
        }
    }

    onStateChanged: if(state === "") { Qt.inputMethod.commit(); inputLoader.endLineEdit() }
}
