import QtQuick 2.3
import QtQuick.Window 2.2
import Fly.Components 1.0

CustomWindow {
    id: window
    x: 0; y: 0
    width: root.width; height: root.height
    minimumHeight: Screen.desktopAvailableHeight; maximumHeight: minimumHeight
    minimumWidth: Screen.desktopAvailableWidth; maximumWidth: minimumWidth
    color: "transparent"
    flags: Qt.FramelessWindowHint
    visible: false

    property alias flyPageStack: flyPageStack
    property alias initialPage: flyPageStack.initialPage

    function appear() {
        if(typeof(compositor) == "undefined" || !compositor) {
            show();
            raise();
        } else {
            compositor.activateWindow(window);
        }
    }

    function disappear() { // @animation
        //disappearAnim.start()
        window.hide()
    }

    Connections { // TODO should connect to dbus, not root
        target: root
        //onCloseChildWindow: if(wid === windowId) { flyPageStack.clearBackStack(); window.hide() }
        onUpdateWidForChildWindow: {
            if(title !== winTitle) return;

            if(window.wid !== windowId) window.wid = windowId;
            root.assignChildWindowToPager(window, windowId);
        }
    }

    Timer { // ORIENTATION BUG
        id: contentItemGeometryUpdate
        interval: 1000
        onTriggered: {
            if(contentItem) {
                if(contentItem.width !== width) contentItem.width = width;
                if(contentItem.height !== height) contentItem.height = height;
                if(flyPageStack.width !== width) flyPageStack.width = width;
                if(flyPageStack.height !== height) flyPageStack.height = height;
            }
        }
    }

    FlyPageStack { id: flyPageStack; width: parent.width; height: parent.height; isMain: false; enabled: window.opacity >= 1.0 }

    // @animation
    //NumberAnimation { id: appearAnim; target: window; property: "opacity"; from: 0; to: 1.0; duration: 400 }
//    SequentialAnimation {
//        id: disappearAnim
//        NumberAnimation { target: window; property: "opacity"; from: 1.0; to: 0; duration: 400 }
//        ScriptAction { script: (function() { flyPageStack.clearBackStack(); window.hide() })() }
//    }

    onVisibleChanged: {
        if(visible) { // @animation
            //appearAnim.start()
        } else {
            flyPageStack.clearBackStack();
        }
    }

    onActiveChanged: {
        if(active && visible) { // sometimes, when window hides, active become true
            root.customPageStack = flyPageStack
            if(root.virtualKeyboard) root.virtualKeyboard.parent = contentItem
            root.selection.parent = contentItem
        } else {
            root.customPageStack = null
            if(root.virtualKeyboard) root.virtualKeyboard.parent = root.contentItem
            root.selection.parent = root.contentItem
        }
    }

    onClosing: {
        if(flyPageStack.backStackFunctions.length === 0) {
            root.customPageStack = null
            if(root.virtualKeyboard) root.virtualKeyboard.parent = root.contentItem
            root.selection.parent = root.contentItem
            disappear()
        } else {
            flyPageStack.processBackClick()
        }
    }

    Screen.onPrimaryOrientationChanged: contentItemGeometryUpdate.restart() // ORIENTATION BUG
    Screen.onDesktopGeometryChanged: contentItemGeometryUpdate.restart()

    Component.onCompleted: {
        console.assert(initialPage, "Error! initialPage undefined!")

        flyPageStack.push("initialPage")
    }
}
